/*
 * BINARY TREES
 * AND
 * RED-BLACK TREES
 *
 * Note: Ta dedomena exoun prosarmostei
 *       katallhla sxetika me to project
 *		 peri analytikhs bathmologias.
 *
 *
 * Project Started : 10th July 2001.
 * Project Finished: 10th August 2001.
 *
 * I was on vacation:
 *   14 - 18 July: Paros.
 *   20 - 26 July: Skiathos.
 *
 */

#include "rb-trees.h"
#include "prep_defs.h"	// Necessary preproccessor directives.
#include "menus.h"
#include "quicksort.h"
#include "display_tree.h"
#include "lists.h"
#include "average.h"

/*********************************************************/

struct owner_properties {
	char firstname [FIRSTNAME_BUFFER];
	char lastname [LASTNAME_BUFFER];
	char id [ID_BUFFER];
	char entry_date [ENTRYDATE_BUFFER];		// dd/mm/yyyy\0
	char birthday [BIRTHDAY_BUFFER];		// dd/mm/yyyy\0
};

typedef struct owner_properties OWNER;
typedef OWNER *OWNERPTR;

/*********************************************************/

int read_data_from_file (FILE *input, TREEPTR Tree, OWNERPTR owner, unsigned int *elements);
int write_owner_to_file (FILE *outputFile, OWNERPTR owner);

/*********************************************************/

int get_owner_properties (OWNERPTR owner);
int owner_properties_verification (OWNERPTR owner);

/*********************************************************/

int get_record_data (DATA_TYPEPTR NewData);
int valid_barythta (char *s);
int data_record_verification (DATA_TYPEPTR newdata);

/*********************************************************/

int insert_operation (TREEPTR Tree, unsigned int *elements);
KEY_TYPE key_delete_menu (TREEPTR Tree);
KEY_TYPE get_code_menu (TREEPTR Tree);
KEY_TYPE get_code_menu_update (TREEPTR Tree);
int key_delete_operation (TREEPTR Tree, KEY_TYPE code, unsigned int *elements);
int traverse_and_delete (TREEPTR Tree, unsigned int *elements);
char delete_record_verification (TREEPTR Tree, TREE_NODEPTR node);

/*********************************************************/

int update_file (char *FileName, TREEPTR Tree, OWNERPTR owner);
int secure_update (char *FileName, TREEPTR Tree, OWNERPTR owner);
int write_INORDER_tree_node_data_to_file (FILE *outputFile, TREEPTR Tree, TREE_NODEPTR x);
int write_POSTORDER_tree_node_data_to_file (FILE *outputFile, TREEPTR Tree, TREE_NODEPTR x);

/*********************************************************/

int key_update_operation (TREEPTR Tree, KEY_TYPE code);
int traverse_and_update (TREEPTR Tree);
char update_record_verification (TREEPTR Tree, TREE_NODEPTR node);
int search_operation (TREEPTR Tree, KEY_TYPE code);

/*********************************************************/

int show_records (TREEPTR Tree);
void present_record (TREEPTR Tree, TREE_NODEPTR node);

/*********************************************************/

int period_average (TREEPTR Tree);
int period_lessons (TREEPTR Tree);
int average_fluctuation (TREEPTR Tree, const int tomeas);
void insert_year (int year, int array_of_years [ARRAY_SIZE]);
int belongs (int something, int array_of_something [ARRAY_SIZE]);
int find_lesson_keys (int xronia, TREEPTR Tree, KEY_TYPE keys [ARRAY_SIZE]);
void insert_key (KEY_TYPE key, KEY_TYPE array_of_keys [ARRAY_SIZE]);
int lesson_grades_stats (TREEPTR Tree, OWNERPTR owner, FILE *somewhere, char *barythta);
int number_of_lessons_with_grade (TREEPTR Tree, double bathmos);
int number_of_lessons_with_grade_and_certain_barythta (TREEPTR Tree, double bathmos, char *baryt);
int number_of_lessons_with_grade_certain_barythta_and_section (TREEPTR Tree, double bathmos, 
															   char *baryt, const int tomeas);
int number_of_lessons_with_grade_and_section (TREEPTR Tree, double bathmos, const int tomeas);

/*********************************************************/

int create_text_with_average_stats (TREEPTR Tree, OWNERPTR owner, char *FileName);
int create_text_with_average_fluctuation (TREEPTR Tree, const int tomeas, OWNERPTR owner, char *FileName);
int create_text_with_lesson_stats (TREEPTR Tree, OWNERPTR owner, char *FileName);
int create_text_with_tree_keys (TREEPTR Tree, OWNERPTR owner, char *FileName);
int create_text_with_tree_colors (TREEPTR Tree, OWNERPTR owner, char *FileName);
int create_subtree (FILE *, TREEPTR, TREE_NODEPTR, int, const int);
int create_text_with_tree (TREEPTR Tree, OWNERPTR owner, char *FileName);
int create_subtree_keys_colors (FILE *, TREEPTR, TREE_NODEPTR, int);

/*********************************************************/

int create_analytikh_textfile (char *FileName, OWNERPTR owner, TREEPTR RB_Tree);

/*********************************************************/

int write_average (AVERAGE_REPLYPTR average, OWNERPTR owner, FILE *somewhere);

/*********************************************************/

int main ()
{
	/*********************************************************/

	/* TREE VARIABLES */
	unsigned int number_of_tree_elements = 0;
	TREEPTR RB_Tree;

	/* OTHER VARIABLES */
	int main_selection = 0;
	KEY_TYPE secondary_selection = 0;
	KEY_TYPE third_selection = 0;
	int result;
	KEY_TYPE code;		// KEY used in Search operations.
	int create = -1;	// Initialize to -1 (not returned by "file_creation_menu" function).
	char temp [REPLY_BUFFER];
	AVERAGE_REPLYPTR average;
	int needs_update, overall_update;
	int wants_to_exit;

	/* VARIABLES NECESSARY FOR FILE PROCESSING */
	FILE *file, *fd;
	OWNERPTR student = (OWNERPTR) malloc (sizeof (OWNER));
	int read_successfully = -1;
	//char FileName [FILENAME_BUFFER];

	/*********************************************************/

	RB_Tree = TREE__Create ();	// Allocate memory for a new RED-BLACK Tree.

	file = fopen (DEFAULT_DATA_FILENAME, "r");

	if (file == NULL) {
		create = file_creation_menu ();
	}

	switch (create) {
		case -1:			// File is open and ready to be read.
			show_logo ();
			printf ("Reading data from file. Please wait...");
			fflush (stdout);
			read_successfully = read_data_from_file (file, RB_Tree, student, 
														&number_of_tree_elements);
			fclose (file);
			break;
		case 0:
			exit (1);		// Exit Program (1).
			break;
		case 1:				// Create a new file.
			get_owner_properties (student);
			file = fopen (DEFAULT_DATA_FILENAME, "w");
			if (file == NULL) {
				printf ("\nFile creation operation failed!\n\a");
				fflush (stdout);
				exit (2);	// Exit Program (2).
			}
			write_owner_to_file (file, student);
			fclose (file);
			break;
	}

	needs_update = 0;
	overall_update = 0;
	wants_to_exit = 0;
	while (main_selection = main_menu (number_of_tree_elements)) {
		switch (main_selection) {

			case INSERT_RECORD:
				while ((secondary_selection = insert_menu ()) != EXIT_INSERT) {
					needs_update = insert_operation (RB_Tree, &number_of_tree_elements);
				}
				break;

			case DELETE_RECORD:
				while ((secondary_selection = delete_menu ()) != EXIT_DELETE) {
					switch (secondary_selection) {

						case KEY_DELETE:
							code = key_delete_menu (RB_Tree);
							if (code == EXIT_DELETE)
								continue;
							needs_update = key_delete_operation (RB_Tree, code, &number_of_tree_elements);
							break;

						case TRAVERSE_AND_DELETE:
							result = traverse_and_delete (RB_Tree, &number_of_tree_elements);
							if (!result) {
								printf ("\n");
								printf ("No Delete operation took place!\n\a");
								printf ("Press ENTER key to return to Delete Menu...");
								fflush (stdout);
								fflush (stdin);
								gets (temp);
							}
							needs_update = result;
							break;

						case EXIT_DELETE:
							/* I will never reach this point. */
							break;
					}
				}
				break;

			case UPDATE_RECORD:
				while ((secondary_selection = update_menu ()) != EXIT_UPDATE) {
					switch (secondary_selection) {
						
						case KEY_UPDATE:
							code = get_code_menu_update (RB_Tree);
							if (code == EXIT_UPDATE)
								continue;
							needs_update = key_update_operation (RB_Tree, code);
							break;

						case TRAVERSE_AND_UPDATE:
							result = traverse_and_update (RB_Tree);
							if (!result) {
								printf ("\n");
								printf ("No Update operation took place!\n\a");
								printf ("Press ENTER key to return to Update Menu...");
								fflush (stdout);
								fflush (stdin);
								gets (temp);
							}
							needs_update = result;
							break;

						case EXIT_UPDATE:
							/* I will never reach this point. */
							break;
					}
				}
				break;

			case SEARCH:
				while ((secondary_selection = search_menu ()) != EXIT_SEARCH) {
					code = get_code_menu (RB_Tree);
					if (code == EXIT_SEARCH)
						continue;
					search_operation (RB_Tree, code);
				}
				break;

			case SHOW_ALL_RECORDS:
				while ((secondary_selection = show_all_menu ()) != EXIT_SHOW_ALL) {
					result = show_records (RB_Tree);
					printf ("\n\n");
					printf ("Press ENTER key to return to Show All Menu...");
					fflush (stdin);
					fflush (stdout);
					gets (temp);
				}
				break;

			case DISPLAY_TREE:
				while ((secondary_selection = tree_representation_menu ()) != EXIT_DISPLAY) {
					switch (secondary_selection) {
						
						case KEY_DISPLAY:
							display_tree_with_keys (RB_Tree);
							printf ("\n\n");
							printf ("Press ENTER key to return to Tree Representation Menu...");
							fflush (stdin);
							fflush (stdout);
							gets (temp);
							break;
						
						case COLOR_DISPLAY:
							display_tree_with_colors (RB_Tree);
							printf ("\n\n");
							printf ("Press ENTER key to return to Tree Representation Menu...");
							fflush (stdin);
							fflush (stdout);
							gets (temp);
							break;

						case EXIT_DISPLAY:
							/* I will never reach this point. */
							break;
					}
				}
				break;

			case CALCULATE_AVERAGE:
				average = calculate_average (RB_Tree);
				CLEARSCREEN;
				write_average (average, student, stdout);
				printf ("\n\n\n");
				printf ("Press ENTER key to return to Main Menu...");
				fflush (stdout);
				fflush (stdin);
				fgets (temp, REPLY_BUFFER, stdin);

				/* Dealloccate unused memory */
				free (average->prwtos_tomeas);
				free (average->deyteros_tomeas);
				free (average->tritos_tomeas);
				free (average->uniform);
				free (average);
				break;

			case DISPLAY_STATISTICS:
				while ((secondary_selection = statistics_menu ()) != EXIT_STATISTICS) {
					switch (secondary_selection) {
						case PERIOD_AVERAGE:
							period_average (RB_Tree);
							break;
						case AVERAGE_FLUCTUATION:
							while ((third_selection = fluctuation_menu ()) != EXIT_FLUCTUATION_MENU) {
								switch (third_selection) {
									case DISPLAY_UNISECTION_FLUCTUATION:
										average_fluctuation (RB_Tree, 0);
										break;
									case DISPLAY_FIRST_SECTION_FLUCTUATION:
										average_fluctuation (RB_Tree, 1);
										break;
									case DISPLAY_SECOND_SECTION_FLUCTUATION:
										average_fluctuation (RB_Tree, 2);
										break;
									case DISPLAY_THIRD_SECTION_FLUCTUATION:
										average_fluctuation (RB_Tree, 3);
										break;
								}
							}
							break;
						case PERIOD_LESSONS:
							period_lessons (RB_Tree);
							break;
						case LESSONS_GRADES:
							while ((third_selection = lesson_grades_menu ()) != EXIT_LESSON_GRADES_MENU) {
								CLEARSCREEN;
								switch (third_selection) {
									case YPOXREWTIKA_MA8HMATA:
										lesson_grades_stats (RB_Tree, student, stdout, "YP");
										break;
									case BASIKA_MA8HMATA:
										lesson_grades_stats (RB_Tree, student, stdout, "BK");
										break;
									case EPILOGHS_MA8HMATA:
										lesson_grades_stats (RB_Tree, student, stdout, "EP");
										break;
									case OLA_TA_MA8HMATA:
										lesson_grades_stats (RB_Tree, student, stdout, "ALL");
										break;
								}
								printf ("\n\n");
								printf ("Press ENTER key to return to Lessons Per Grade Menu...");
								fflush (stdout);
								fflush (stdin);
								gets (temp);
							}
							break;
						case EXIT_STATISTICS:
							// I should never reach this point.
							break;
					}
				}
				break;

			case CREATE_TEXTFILE:
				while ((secondary_selection = create_files_menu ()) != EXIT_CREATE) {
					switch (secondary_selection) {

						case CREATE_ANALYTIKH:
							result = create_analytikh_textfile (DEFAULT_TEXT_FILENAME, student, RB_Tree);
							CLEARSCREEN;
							show_logo ();
							if (result) {
								printf ("Text File was created successfully!\n");
							}
							else {
								printf ("Error occured during file creation!\n");
								printf ("File was not created.\n\a");
							}
							printf ("\n\n");
							printf ("Press ENTER key to return to File Creation Menu...");
							fflush (stdout);
							fflush (stdin);
							gets (temp);
							break;

						case CREATE_AVERAGE:
							CLEARSCREEN;
							show_logo ();
							average = calculate_average (RB_Tree);
							if ((fd = fopen (DEFAULT_AVERAGE_FILENAME, "w")) == NULL) {
								printf ("Error occured during file creation!\n");
								printf ("File was not created.\n\a");
							}
							else {
								write_average (average, student, fd);
								fclose (fd);
								printf ("Text File was created successfully!\n");
							}
							printf ("\n\n");
							printf ("Press ENTER key to return to File Creation Menu...");
							fflush (stdout);
							fflush (stdin);
							gets (temp);

							/* Dealloccate unused memory */
							free (average->prwtos_tomeas);
							free (average->deyteros_tomeas);
							free (average->tritos_tomeas);
							free (average->uniform);
							free (average);
							break;

						case CREATE_LESSON_STATS:
							result = create_text_with_lesson_stats (RB_Tree, student, DEFAULT_LESSON_STATS_FILENAME);
							CLEARSCREEN;
							show_logo ();
							if (result) {
								printf ("Text File was created successfully!\n");
							}
							else {
								printf ("Error occured during file creation!\n");
								printf ("File was not created.\n\a");
							}
							printf ("\n\n");
							printf ("Press ENTER key to return to File Creation Menu...");
							fflush (stdout);
							fflush (stdin);
							gets (temp);
							break;

						case CREATE_AVERAGE_PERIOD:
							result = create_text_with_average_stats (RB_Tree, student, DEFAULT_AVERAGE_STATS_FILENAME);
							CLEARSCREEN;
							show_logo ();
							if (result) {
								printf ("Text File was created successfully!\n");
							}
							else {
								printf ("Error occured during file creation!\n");
								printf ("File was not created.\n\a");
							}
							printf ("\n\n");
							printf ("Press ENTER key to return to File Creation Menu...");
							fflush (stdout);
							fflush (stdin);
							gets (temp);
							break;

						case CREATE_AVERAGE_FLUCTUATION:
							while ((third_selection = create_fluctuation_menu ()) != EXIT_FLUCTUATION) {
								switch (third_selection) {
									case UNISECTION_FLUCTUATION:
										result = create_text_with_average_fluctuation (RB_Tree, 0, student, DEFAULT_UNISECTION_AVERAGE_FLUCTUATION_FILENAME);
										break;
									case FIRST_SECTION_FLUCTUATION:
										result = create_text_with_average_fluctuation (RB_Tree, 1, student, DEFAULT_FIRST_SECTION_AVERAGE_FLUCTUATION_FILENAME);
										break;
									case SECOND_SECTION_FLUCTUATION:
										result = create_text_with_average_fluctuation (RB_Tree, 2, student, DEFAULT_SECOND_SECTION_AVERAGE_FLUCTUATION_FILENAME);
										break;
									case THIRD_SECTION_FLUCTUATION:
										result = create_text_with_average_fluctuation (RB_Tree, 3, student, DEFAULT_THIRD_SECTION_AVERAGE_FLUCTUATION_FILENAME);
										break;
								}
								CLEARSCREEN;
								show_logo ();
								if (result) {
									printf ("Text File was created successfully!\n");
								}
								else {
									printf ("Error occured during file creation!\n");
									printf ("File was not created.\n\a");
								}
								printf ("\n\n");
								printf ("Press ENTER key to return to Create Average Grade Fluctuation File Menu...");
								fflush (stdout);
								fflush (stdin);
								gets (temp);
							}
							break;

						case CREATE_LESSONS_PER_GRADE:
							while ((third_selection = create_lesson_grades_menu ()) != EXIT_CREATE_LESSON_GRADES_MENU) {
								CLEARSCREEN;
								show_logo ();
								switch (third_selection) {
									case CREATE_YPOXREWTIKA_MA8HMATA:
										fd = fopen (DEFAULT_YPOXREWTIKA_PER_GRADE_FILENAME, "w");
										if (fd != NULL) {
											result = lesson_grades_stats (RB_Tree, student, fd, "YP");
											printf ("Text File was created successfully!\n");
											fclose (fd);
										}
										else {
											printf ("Error occured during file creation!\n");
											printf ("File was not created.\n\a");
										}
										break;
									case CREATE_BASIKA_MA8HMATA:
										fd = fopen (DEFAULT_BASIKA_PER_GRADE_FILENAME, "w");
										if (fd != NULL) {
											result = lesson_grades_stats (RB_Tree, student, fd, "BK");
											printf ("Text File was created successfully!\n");
											fclose (fd);
										}
										else {
											printf ("Error occured during file creation!\n");
											printf ("File was not created.\n\a");
										}
										break;
									case CREATE_EPILOGHS_MA8HMATA:
										fd = fopen (DEFAULT_EPILOGHS_PER_GRADE_FILENAME, "w");
										if (fd != NULL) {
											result = lesson_grades_stats (RB_Tree, student, fd, "EP");
											printf ("Text File was created successfully!\n");
											fclose (fd);
										}
										else {
											printf ("Error occured during file creation!\n");
											printf ("File was not created.\n\a");
										}
										break;
									case CREATE_OLA_TA_MA8HMATA:
										fd = fopen (DEFAULT_LESSONS_PER_GRADE_FILENAME, "w");
										if (fd != NULL) {
											result = lesson_grades_stats (RB_Tree, student, fd, "ALL");
											printf ("Text File was created successfully!\n");
											fclose (fd);
										}
										else {
											printf ("Error occured during file creation!\n");
											printf ("File was not created.\n\a");
										}
										break;
								}
								printf ("\n\n");
								printf ("Press ENTER key to return to Create Lessons Per Grade Files Menu...");
								fflush (stdout);
								fflush (stdin);
								gets (temp);
							}
							break;

						case CREATE_TREE_KEYS:
							result = create_text_with_tree_keys (RB_Tree, student, DEFAULT_TREE_KEYS_FILENAME);
							CLEARSCREEN;
							show_logo ();
							if (result) {
								printf ("Text File was created successfully!\n");
							}
							else {
								printf ("Error occured during file creation!\n");
								printf ("File was not created.\n\a");
							}
							printf ("\n\n");
							printf ("Press ENTER key to return to File Creation Menu...");
							fflush (stdout);
							fflush (stdin);
							gets (temp);
							break;

						case CREATE_TREE_COLORS:
							result = create_text_with_tree_colors (RB_Tree, student, DEFAULT_TREE_COLORS_FILENAME);
							CLEARSCREEN;
							show_logo ();
							if (result) {
								printf ("Text File was created successfully!\n");
							}
							else {
								printf ("Error occured during file creation!\n");
								printf ("File was not created.\n\a");
							}
							printf ("\n\n");
							printf ("Press ENTER key to return to File Creation Menu...");
							fflush (stdout);
							fflush (stdin);
							gets (temp);
							break;

						case CREATE_TREE:
							result = create_text_with_tree (RB_Tree, student, DEFAULT_TREE_FILENAME);
							CLEARSCREEN;
							show_logo ();
							if (result) {
								printf ("Text File was created successfully!\n");
							}
							else {
								printf ("Error occured during file creation!\n");
								printf ("File was not created.\n\a");
							}
							printf ("\n\n");
							printf ("Press ENTER key to return to File Creation Menu...");
							fflush (stdout);
							fflush (stdin);
							gets (temp);
							break;

						case CREATE_GENERAL_HELP:
							CLEARSCREEN;
							show_logo ();
							if ((fd = fopen (DEFAULT_GENERAL_HELP_FILENAME, "w")) == NULL) {
								printf ("Error occured during file creation!\n");
								printf ("File was not created.\n\a");
							}
							else {
								general_screen (fd);
								fclose (fd);
								printf ("Text File was created successfully!\n");
							}
							printf ("\n\n");
							printf ("Press ENTER key to return to File Creation Menu...");
							fflush (stdout);
							fflush (stdin);
							gets (temp);
							break;

						case CREATE_MODIFICATIONS:
							CLEARSCREEN;
							show_logo ();
							if ((fd = fopen (DEFAULT_MODIFICATIONS_FILENAME, "w")) == NULL) {
								printf ("Error occured during file creation!\n");
								printf ("File was not created.\n\a");
							}
							else {
								modifications_screen (fd);
								fclose (fd);
								printf ("Text File was created successfully!\n");
							}
							printf ("\n\n");
							printf ("Press ENTER key to return to File Creation Menu...");
							fflush (stdout);
							fflush (stdin);
							gets (temp);
							break;

						case EXIT_CREATE:
							// I should never reach this point.
							break;
					}
				}
				break;

			case UPDATE_DATAFILE:
				while ((secondary_selection = update_datafile_menu ()) != EXIT_UPDATE_DATAFILE) {
					switch (secondary_selection) {
						case FAST_UPDATE:
							result = update_file (DEFAULT_DATA_FILENAME, RB_Tree, student);
							break;

						case SECURE_UPDATE:
							result = secure_update (DEFAULT_DATA_FILENAME, RB_Tree, student);
							break;

						case EXIT_UPDATE_DATAFILE:
							break;
					}
					CLEARSCREEN;
					show_logo ();
					if (result) {
						printf ("Data File was updated successfully!\n");
						overall_update = 0;
						needs_update = 0;	// Indicate that nothing needs to be updated.
					}
					else {
						printf ("Error occured during update operation!\n");
						printf ("If this was a secure update operation do the following:\n");
						printf (" 1. Exit Program.\n");
						printf (" 2. Rename \"grades.old\" file to \"grades.dat\" file.\n");
						printf (" 3. Re-run program and make sure that \"grades.dat\" file is not open!\n");
					}
					printf ("\n\n");
					printf ("Press ENTER key to return to Update DataFile Menu...");
					fflush (stdout);
					fflush (stdin);
					gets (temp);
				}
				break;

			case HELP:
				while ((secondary_selection = help_menu()) != EXIT_HELP) {
					switch (secondary_selection) {
						case GENERAL:
							CLEARSCREEN;
							general_screen (stdout);
							break;
						case ABOUT:
							about_screen ();
							break;
						case MODIFICATIONS:
							CLEARSCREEN;
							modifications_screen (stdout);
							break;
						case EXIT_HELP:
							//I will never reach this point.
							break;
					}
					printf ("\n\n");
					printf ("Press ENTER key to return to Help Menu...");
					fflush (stdout);
					fflush (stdin);
					gets (temp);
				}
				break;

			case EXIT_PROGRAM:
				if (overall_update == 0)
					wants_to_exit = 1;	//Indicate user wants to exit.
				else {
					result = verify_exit ();
					if (result)
						wants_to_exit = 1;	//Indicate user wants to exit.
					else
						wants_to_exit = 0;
				}
				break;

		}

		if ((overall_update == 0) && (needs_update == 1))
			overall_update = 1;

		if (wants_to_exit)
			break;
	}

	return 0;	// Procceed to successful finish.
}

/*********************************************************/

int show_records (TREEPTR Tree)
{
	int successor;
	char reply;
	char reply_string [REPLY_BUFFER];
	TREE_NODEPTR current, min, max, temp;

	min = TREE__Minimum (Tree, Tree->ROOT);
	current = min;
	max = TREE__Maximum (Tree, Tree->ROOT);
	temp = Tree->NIL;

	do {
		present_record (Tree, current);
		if (current == min) {
			printf (" N = Next record\n");
			printf (" L = Leave Show All Records operation\n");
		}
		else if (current == max) {
			printf (" P = Previous record\n");
			printf (" L = Leave Show All Records operation\n");
		}
		else {
			printf (" P = Previous record\t N = Next record\n");
			printf (" L = Leave Show All Records operation\n");
		}
		printf (" ? ");
		fflush (stdout);
		fflush (stdin);
		gets (reply_string);
		if (strlen (reply_string) != 1)
			reply = 'a';
		else
			reply = reply_string [0];
		successor = 0;
		if ((reply == 'n') || (reply == 'N')) {
			temp = TREE__Successor (Tree, current);
			successor = 1;
		}
		if ((reply == 'p') || (reply == 'P')) {
			temp = TREE__Predecessor (Tree, current);
			successor = -1;
		}
		if (temp != Tree->NIL)
			current = temp;
	} while ((reply != 'l') && (reply != 'L'));

	return 1;	// Indicate end of Tree traversing.
}

void present_record (TREEPTR Tree, TREE_NODEPTR node)
{
	CLEARSCREEN;
	show_logo ();
//	if (node == Tree->ROOT) {
//		printf ("This is the ROOT\n");
//	}
	printf ("Data Fields\n");
	printf ("-----------\n");
	printf ("NAME      : %s\n", node->data->lesson);
	printf ("CODE      : %ld\n", node->data->code);
	printf ("GRADE     : %.1f\n", node->data->grade);
	printf ("DATE      : %s\n", node->data->date_passed);
	printf ("BARYTHTA  : %s\n", node->data->barythta);
	printf ("ACAD. YEAR: %s\n", node->data->akadhmaiko_etos);
	printf ("SECTION   : %d\n", node->data->section);
	printf ("SEMESTER  : %d\n", node->data->semester);
	printf ("\n\n");
/*
	printf ("Node Fields\n");
	printf ("-----------\n");
	printf ("Node Color: %s\n", node->color == BLACK ? "BLACK" : "RED");
	printf ("Node Parent: %s (%d)\n", node->parent->data->lesson, node->parent->key);
	printf ("Left Child: %s (%d)\n", node->left_child->data->lesson, node->left_child->key);
	printf ("Right Child: %s (%d)\n", node->right_child->data->lesson, node->right_child->key);
*/
}

int read_data_from_file (FILE *inputFile, TREEPTR Tree, OWNERPTR owner, unsigned int *elements)
{
	TREE_NODEPTR Newnode;
	int inserted;
	int errors_encountered = 0;
	char temp [2];

	/*** Read owner characteristics. ***/
	fgets ((char *) owner, (sizeof (OWNER) + 1), inputFile);

//  DOKIMH AN DIABAZEI SWSTA....
// 	printf ("\nI read: \n");
//	printf ("F : %s\n", owner->firstname);
//	printf ("L : %s\n", owner->lastname);
//	printf ("ID: %s\n", owner->id);
//	printf ("E : %s\n", owner->entry_date);
//	printf ("B : %s\n", owner->birthday);
//	scanf ("%d", &inserted);
//	TELOS DOKIMHS...

	while (!feof(inputFile)) {
		//CLEARSCREEN;
		//show_logo ();
		/*** Create new node to store data. ***/
		Newnode = TREE_NODE__Create (Tree);
//		printf ("Memory allocated for new node.\n");
//		printf ("Memory also allocated for data structure in new node.\n");
//		fflush (stdout);
		/*** Read lessons passed. ***/
		fread (Newnode->data, sizeof (DATA_TYPE), 1, inputFile);
		Newnode->key = Newnode->data->code;
//		printf ("Data Fields\n");
//		printf ("-----------\n");
//		printf ("Name: %s\n", Newnode->data->lesson);
//		printf ("Code: %ld%\n", Newnode->data->code);
//		printf ("Grde: %f%\n", Newnode->data->grade);
//		printf ("Date: %s\n", Newnode->data->date_passed);
//		printf ("Bart: %s\n", Newnode->data->barythta);
//		printf ("AcYr: %s\n", Newnode->data->akadhmaiko_etos);
//		printf ("Sect: %d\n", Newnode->data->section);
//		printf ("Semt: %d\n", Newnode->data->semester);
//		scanf ("%d", &inserted);
		if (in_range (LOWER_BOUND_CODE, UPPER_BOUND_CODE, Newnode->key)){
			inserted = RED_BLACK__Insert (Tree, Newnode);
			if (inserted)
				*elements = (*elements) + 1;
			else {
				printf ("\nError inserting node!\n\a\n");
				scanf ("%d", &inserted);
				errors_encountered++;
			}
		}
		else {
/*
			printf ("Data Fields\n");
			printf ("-----------\n");
			printf ("Name: %s\n", Newnode->data->lesson);
			printf ("Code: %ld%\n", Newnode->data->code);
			printf ("Grde: %f%\n", Newnode->data->grade);
			printf ("Date: %s\n", Newnode->data->date_passed);
			printf ("Bart: %s\n", Newnode->data->barythta);
			printf ("AcYr: %s\n", Newnode->data->akadhmaiko_etos);
			printf ("Sect: %d\n", Newnode->data->section);
			printf ("Semt: %d\n", Newnode->data->semester);
*/
			/* Delete useless data stored in memory */
			inserted = TREE_NODE__Destroy (Newnode);
			if (inserted) {
				//printf ("\n\n\n\n");
				//printf ("Memory kept for useless node was\n");
				//printf ("SUCCESSFULLY DE-ALLOCATED.\n\n");
				//printf ("Press ENTER key to continue...");
				//gets (temp);
				;
			}
			else {
				printf ("\n\n\n\n");
				printf ("Allocated memory for a useless node.\n\n");
				printf ("Press ENTER key to continue...");
				fflush (stdout);
				fflush (stdin);
				gets (temp);
				errors_encountered++;
			}
		}
	}

	/*
	 * Indicate how many errors occured:
	 * return 0 => Everything is OK.
	 * return > 0 => Something went wrong.
	 */

	return errors_encountered;
}

int write_owner_to_file (FILE *arxeio, OWNERPTR owner)
{
	fwrite (owner, sizeof (OWNER), 1, arxeio);

	return 1;	/* Indicate successful insertion. */
}

/*********************************************************/

int get_owner_properties (OWNERPTR owner)
{
	int verify = 0;
	char temp [2 * REPLY_BUFFER];

	do {
		CLEARSCREEN;
		show_logo ();
		printf ("OWNER PROPERTIES\n");
		printf ("----------------\n");
		printf ("Please identify yourself:\n");
		do {
			printf ("Firstname (max %d characters): ", (FIRSTNAME_BUFFER - 1));
			gets (temp);
		} while (strlen (temp) > (FIRSTNAME_BUFFER - 1));
		strcpy (owner->firstname, temp);
		do {
			printf ("Lastname (max %d characters): ", (LASTNAME_BUFFER - 1));
			gets (temp);
		} while (strlen (temp) > (LASTNAME_BUFFER - 1));
		strcpy (owner->lastname, temp);
		do {
			printf ("Identification Number (max %d characters): ", (ID_BUFFER - 1));
			gets (temp);
		} while (strlen (temp) > (ID_BUFFER - 1));
		strcpy (owner->id, temp);
		do {
			printf ("Entry Date (dd/mm/yyyy - %d characters): ", (ENTRYDATE_BUFFER - 1));
			gets (temp);
		} while (strlen (temp) > (ENTRYDATE_BUFFER - 1));
		strcpy (owner->entry_date, temp);
		do {
			printf ("Birthday (dd/mm/yyyy - %d characters): ", (BIRTHDAY_BUFFER - 1));
			gets (temp);
		} while (strlen (temp) > (BIRTHDAY_BUFFER - 1));
		strcpy (owner->birthday, temp);

		verify = owner_properties_verification (owner);
	} while (verify != 1);

	return 1;	// Indicate succesful insertion.
}

int owner_properties_verification (OWNERPTR owner)
{
	char reply_string [REPLY_BUFFER];
	char reply = 'a';

	do {
		CLEARSCREEN;
		show_logo ();
		printf ("OWNER PROPERTIES VERIFICATION\n");
		printf ("-----------------------------\n");
		printf ("Firstname : %s\n", owner->firstname);
		printf ("Lastname  : %s\n", owner->lastname);
		printf ("ID Number : %s\n", owner->id);
		printf ("Entry Date: %s\n", owner->entry_date);
		printf ("Birthday  : %s\n", owner->birthday);
		printf ("\nAre the above fields correct?\n");
		printf ("(y = YES, n = NO) ? ");
		fflush (stdout);
		fflush (stdin);
		gets (reply_string);
		if (strlen (reply_string) != 1)
			reply = 'a';
		else
			reply = reply_string [0];
	} while ((reply != 'y') && (reply != 'n') && 
			(reply != 'Y') && (reply != 'N'));

	if ((reply == 'y') || (reply == 'Y'))
		return 1;	// Verify owner properties.
	else
		return 0;	// DO NOT Verify owner properties.
}

/*********************************************************/

int get_record_data (DATA_TYPEPTR NewData)
{
	char temp [4 * REPLY_BUFFER];
	int verify;

	do {
		CLEARSCREEN;
		show_logo ();
		printf ("INSERT NEW DATA\n");
		printf ("---------------\n");
		do {
			printf ("Lesson Code (long integer): ");
			gets (temp);
		} while ((!numeric_int (temp)) || (strlen (temp) > (4 * REPLY_BUFFER - 1)));
		NewData->code = atol (temp);
		do {
			printf ("Lesson Name (max %d characters): ", (LESSON_NAME_BUFFER - 1));
			gets (temp);
		} while (strlen (temp) > (LESSON_NAME_BUFFER - 1));
		strcpy (NewData->lesson, temp);
		do {
			printf ("Grade (double): ");
			gets (temp);
		} while (strlen (temp) > (4 * REPLY_BUFFER - 1));
		NewData->grade = atof (temp);
		do {
			printf ("Date Passed (dd/mm/yyyy - %d characters): ", (DATE_PASSED_BUFFER - 1));
			gets (temp);
		} while (strlen (temp) > (DATE_PASSED_BUFFER - 1));
		strcpy (NewData->date_passed, temp);
		do {
			printf ("Academic Year (yyyy-yy - %d characters): ", (AKADHMAIKO_ETOS_BUFFER - 1));
			gets (temp);
		} while (strlen (temp) > (AKADHMAIKO_ETOS_BUFFER - 1));
		strcpy (NewData->akadhmaiko_etos, temp);
		do {
			printf ("Semester (integer): ");
			gets (temp);
		} while ((!numeric_int (temp)) || (strlen (temp) > (4 * REPLY_BUFFER - 1)));
		NewData->semester = atoi (temp);
		do {
			printf ("Barythta (YP || BK || EP || PT): ");
			gets (temp);
		} while ((strlen (temp) > (BARYTHTA_BUFFER - 1)) || (!valid_barythta (temp)));
		strcpy (NewData->barythta, temp);
		do {
			printf ("Section (integer): ");
			gets (temp);
		} while ((!numeric_int (temp)) || (strlen (temp) > (4 * REPLY_BUFFER - 1)));
		NewData->section = atoi (temp);

		verify = data_record_verification (NewData);
	} while (verify != 1);

	return 1;	// Indicate succesful insertion.
}

int valid_barythta (char *s)
{
	if (strcmp(s, "YP") == 0)
		return 1;	// Valid.
	if (strcmp(s, "BK") == 0)
		return 1;	// Valid.
	if (strcmp(s, "EP") == 0)
		return 1;	// Valid.
	if (strcmp(s, "PT") == 0)
		return 1;	// Valid.

	return 0;		// Invalid.
}

int data_record_verification (DATA_TYPEPTR newdata)
{
	char reply_string [REPLY_BUFFER];
	char reply = 'a';

	do {
		CLEARSCREEN;
		show_logo ();
		printf ("DATA FIELDS VERIFICATION\n");
		printf ("------------------------\n");
		printf ("Lesson Code: %ld\n", newdata->code);
		printf ("Lesson Name: %s\n", newdata->lesson);
		printf ("Grade      : %f\n", newdata->grade);
		printf ("Date Passed: %s\n", newdata->date_passed);
		printf ("Acad. Year : %s\n", newdata->akadhmaiko_etos);
		printf ("Semester   : %d\n", newdata->semester);
		printf ("Barythta   : %s\n", newdata->barythta);
		printf ("Section    : %d\n", newdata->section);
		printf ("\nAre the above fields correct?\n");
		printf ("(y = YES, n = NO) ? ");
		fflush (stdout);
		fflush (stdin);
		gets (reply_string);
		if (strlen (reply_string) != 1)
			reply = 'a';
		else
			reply = reply_string [0];
	} while ((reply != 'y') && (reply != 'n') && 
			(reply != 'Y') && (reply != 'N'));

	if ((reply == 'y') || (reply == 'Y'))
		return 1;	// Verify data fields.
	else
		return 0;	// DO NOT Verify data fields.
}

/*********************************************************/

int insert_operation (TREEPTR Tree, unsigned int *elements)
{
	int result, inserted;
	TREE_NODEPTR Newnode;
	char temp [REPLY_BUFFER];

	Newnode = TREE_NODE__Create (Tree);
	result = get_record_data (Newnode->data);
	if (result == 1) {
		Newnode->key = Newnode->data->code;
		result = RED_BLACK__Insert (Tree, Newnode);
		if (result) {
			*elements = (*elements) + 1;
			inserted = 1;	// Indicate node was inserted successfully.
		}
		else {
			printf ("\nNo insertion took place!\a\n");
			printf ("A lesson with the same code is already inserted.\n");
			printf ("Press ENTER key to return to Insert Menu...");
			fflush (stdout);
			fflush (stdin);
			gets (temp);
			inserted = -1;	// Indicate node was NOT inserted successfully.
		}
	}
	else {
		TREE_NODE__Destroy (Newnode);
		inserted = 0;		// Indicate successful DE_ALLOCATION of memory.
	}

	printf ("Number of tree elements: %u", *elements);
	gets (temp);

	return inserted;	// Indicate result of insert_operation.
}

KEY_TYPE key_delete_menu (TREEPTR Tree)
{
	KEY_TYPE selection;
	TREE_NODEPTR min, max;
	char reply [REPLY_BUFFER];

	min = TREE__Minimum (Tree, Tree->ROOT);
	max = TREE__Maximum (Tree, Tree->ROOT);

	do {
		CLEARSCREEN;
		show_logo ();
		printf ("DELETE A SPECIFIC RECORD\n");
		printf ("------------------------\n");
		printf ("Range of KEYS currently inserted in RED-BLACK Tree: [%ld, %ld]\n", 
				min->key, max->key);
		printf ("Valid KEYS: [%ld, %ld]\n", LOWER_BOUND_CODE, UPPER_BOUND_CODE);
		printf ("Return KEY: %d\n", EXIT_DELETE);
		printf ("Delete a specific record with KEY (lesson code) code: \n");
		printf ("\n");
		printf (" ? ");
		fflush (stdout);
		fflush (stdin);
		gets (reply);
		if (numeric_int (reply))
			selection = atoi (reply);
		else
			selection = 0;
	} while ((!in_range (LOWER_BOUND_CODE, UPPER_BOUND_CODE, selection)) &&
			(selection != EXIT_DELETE));

	return selection;
}

KEY_TYPE get_code_menu (TREEPTR Tree)
{
	KEY_TYPE selection;
	char reply [REPLY_BUFFER];
	TREE_NODEPTR min, max;

	min = TREE__Minimum (Tree, Tree->ROOT);
	max = TREE__Maximum (Tree, Tree->ROOT);

	do {
		CLEARSCREEN;
		show_logo ();
		printf ("SEARCH FOR A SPECIFIC RECORD\n");
		printf ("----------------------------\n");
		printf ("Range of KEYS currently inserted in RED-BLACK Tree: [%ld, %ld]\n", 
				min->key, max->key);
		printf ("Valid KEYS: [%ld, %ld]\n", LOWER_BOUND_CODE, UPPER_BOUND_CODE);
		printf ("Return KEY: %d\n", EXIT_SEARCH);
		printf ("Search for a specific record with KEY (lesson code) code: \n");
		printf ("\n");
		printf (" ? ");
		fflush (stdout);
		fflush (stdin);
		gets (reply);
		if (numeric_int (reply))
			selection = atol (reply);
		else
			selection = 0;
	} while ((!in_range (LOWER_BOUND_CODE, UPPER_BOUND_CODE, selection)) &&
			(selection != EXIT_SEARCH));

	return selection;
}

int key_delete_operation (TREEPTR Tree, KEY_TYPE code, unsigned int *elements)
{
	TREE_NODEPTR NodeReturned;
	char reply;
	char temp [REPLY_BUFFER];
	int result;

	NodeReturned = TREE__Search (Tree, code);
	if (NodeReturned == Tree->NIL) {
		CLEARSCREEN;
		show_logo ();
		printf ("No valid record with the specific key could be found!\n\n\n");
		result = -1;		// Indicate that no record with the specific key could be found.
	}
	else {
		do {
			present_record (Tree, NodeReturned);
			printf ("Do you want to delete this record?\n");
			printf ("(y = YES, n = NO) ? ");
			fflush (stdout);
			fflush (stdin);
			gets (temp);
			if (strlen (temp) != 1)
				reply = 'a';
			else
				reply = temp [0];
		} while ((reply != 'y') && (reply != 'n') && 
				(reply != 'Y') && (reply != 'N'));

		if ((reply == 'y') || (reply == 'Y')) {
			TREE_NODE__Destroy (RED_BLACK__Delete (Tree, NodeReturned));
			(*elements) = (*elements) - 1;
			printf ("\nThe node with its corresponding fields ");
			printf ("was deleted successfully.\n");
			result = 1;		// Indicate that node was deleted successfully.
		}
		else {
			printf ("\nNode is retained.\n");
			result = 0;		// Indicate that node was not destroyed.
		}
	}

	printf ("Press ENTER key to return to Delete Menu...");
	fflush (stdout);
	fflush (stdin);
	gets (temp);

	return result;	// Indicate result of key_delete_operation.
}

int traverse_and_delete (TREEPTR Tree, unsigned int *elements)
{
	int successor, result = -1;
	char reply, verify;
	char reply_string [REPLY_BUFFER];
	char enter [REPLY_BUFFER];
	TREE_NODEPTR current, min, max, temp;

	min = TREE__Minimum (Tree, Tree->ROOT);
	current = min;
	max = TREE__Maximum (Tree, Tree->ROOT);
	temp = Tree->NIL;

	do {
		present_record (Tree, current);
		if (current == min) {
			printf (" N = Next record\n");
			printf (" D = Delete record\t L = Leave Delete operation\n");
		}
		else if (current == max) {
			printf (" P = Previous record\n");
			printf (" D = Delete record\t L = Leave Delete operation\n");
		}
		else {
			printf (" P = Previous record\t N = Next record\n");
			printf (" D = Delete record\t L = Leave Delete operation\n");
		}
		printf (" ? ");
		fflush (stdout);
		fflush (stdin);
		gets (reply_string);
		if (strlen (reply_string) != 1)
			reply = 'a';
		else
			reply = reply_string [0];
		successor = 0;
		if ((reply == 'd') || (reply == 'D')) {
			verify = delete_record_verification (Tree, current);
			if (verify == 'y') {
				result = TREE_NODE__Destroy (RED_BLACK__Delete (Tree, current));
				if (result) {
					*elements = (*elements) - 1;
					printf ("\n");
					printf ("Node with its corresponding fields was deleted succesfully.\n");
					printf ("Press ENTER key to return to Delete Menu...");
					fflush (stdout);
					fflush (stdin);
					gets (enter);
					break;
				}
				else {
					printf ("\n");
					printf ("Node was not deleted!\n\a");
					printf ("Press ENTER key to continue...");
					fflush (stdout);
					fflush (stdin);
					gets (enter);
				}
			}
			verify = 'n';
		}
		if ((reply == 'n') || (reply == 'N')) {
			temp = TREE__Successor (Tree, current);
			successor = 1;
		}
		if ((reply == 'p') || (reply == 'P')) {
			temp = TREE__Predecessor (Tree, current);
			successor = -1;
		}
		if (temp != Tree->NIL)
			current = temp;
	} while ((reply != 'l') && (reply != 'L'));

	/*
	 * return 1 => Something was deleted.
	 * return 0 => Nothing was deleted.
	 */

	if (verify == 'y')
		return 1;
	else
		return 0;
}

char delete_record_verification (TREEPTR Tree, TREE_NODEPTR node)
{
	char reply_string [REPLY_BUFFER];
	char reply;

	do {
		present_record (Tree, node);
		printf ("Are you sure you want to delete this record?\n");
		printf ("(y = YES, n = NO) ? ");
		fflush (stdout);
		fflush (stdin);
		gets (reply_string);
		if (strlen (reply_string) != 1)
			reply = 'a';
		else
			reply = reply_string [0];
	} while ((reply != 'y') && (reply != 'n') && 
			(reply != 'Y') && (reply != 'N'));

	if ((reply == 'y') || (reply == 'Y'))
		return 'y';
	else
		return 'n';
}

KEY_TYPE get_code_menu_update (TREEPTR Tree)
{
	KEY_TYPE selection;
	char reply [REPLY_BUFFER];
	TREE_NODEPTR min, max;

	min = TREE__Minimum (Tree, Tree->ROOT);
	max = TREE__Maximum (Tree, Tree->ROOT);

	do {
		CLEARSCREEN;
		show_logo ();
		printf ("UPDATE A SPECIFIC RECORD\n");
		printf ("------------------------\n");
		printf ("Range of KEYS currently inserted in RED-BLACK Tree: [%ld, %ld]\n", 
				min->key, max->key);
		printf ("Valid KEYS: [%ld, %ld]\n", LOWER_BOUND_CODE, UPPER_BOUND_CODE);
		printf ("Return KEY: %d\n", EXIT_UPDATE);
		printf ("Search for a specific record with KEY (lesson code) code: \n");
		printf ("\n");
		printf (" ? ");
		fflush (stdout);
		fflush (stdin);
		gets (reply);
		if (numeric_int (reply))
			selection = atol (reply);
		else
			selection = 0;
	} while ((!in_range (LOWER_BOUND_CODE, UPPER_BOUND_CODE, selection)) &&
			(selection != EXIT_UPDATE));

	return selection;
}

int traverse_and_update (TREEPTR Tree)
{
	int successor;
	char reply, verified;
	int verify, updated;
	char reply_string [REPLY_BUFFER];
	char temp [4 * REPLY_BUFFER];
	TREE_NODEPTR current, min, max, searchNode, updateNode, tempNode;

	min = TREE__Minimum (Tree, Tree->ROOT);
	current = min;
	max = TREE__Maximum (Tree, Tree->ROOT);
	tempNode = Tree->NIL;

	do {
		present_record (Tree, current);
		if (current == min) {
			printf (" N = Next record\n");
			printf (" U = Update record\t L = Leave Update operation\n");
		}
		else if (current == max) {
			printf (" P = Previous record\n");
			printf (" U = Update record\t L = Leave Update operation\n");
		}
		else {
			printf (" P = Previous record\t N = Next record\n");
			printf (" U = Update record\t L = Leave Update operation\n");
		}
		printf (" ? ");
		fflush (stdout);
		fflush (stdin);
		gets (reply_string);
		if (strlen (reply_string) != 1)
			reply = 'a';
		else
			reply = reply_string [0];
		successor = 0;
		verified = 'n';
		if ((reply == 'u') || (reply == 'U')) {
			verified = update_record_verification (Tree, current);
			if (verified == 'y') {
				updateNode = TREE_NODE__Create (Tree);
				do {
					CLEARSCREEN;
					show_logo ();
					printf ("Old Data Fields\n");
					printf ("---------------\n");
					printf ("NAME      : %s\n", current->data->lesson);
					printf ("CODE      : %ld\n", current->data->code);
					printf ("GRADE     : %.1f\n", current->data->grade);
					printf ("DATE      : %s\n", current->data->date_passed);
					printf ("BARYTHTA  : %s\n", current->data->barythta);
					printf ("ACAD. YEAR: %s\n", current->data->akadhmaiko_etos);
					printf ("SECTION   : %d\n", current->data->section);
					printf ("SEMESTER  : %d\n", current->data->semester);
					printf ("\n\n");
					printf ("Update Data Fileds\n");
					printf ("------------------\n");
					do {
						printf ("Lesson Code (long integer): ");
						gets (temp);
					} while ((!numeric_int (temp)) || (strlen (temp) > (4 * REPLY_BUFFER - 1)));
					updateNode->data->code = atol (temp);
					do {
						printf ("Lesson Name (max %d characters): ", (LESSON_NAME_BUFFER - 1));
						gets (temp);
					} while (strlen (temp) > (LESSON_NAME_BUFFER - 1));
					strcpy (updateNode->data->lesson, temp);
					do {
						printf ("Grade (double): ");
						gets (temp);
					} while (strlen (temp) > (4 * REPLY_BUFFER - 1));
					updateNode->data->grade = atof (temp);
					do {
						printf ("Date Passed (dd/mm/yyyy - %d characters): ", (DATE_PASSED_BUFFER - 1));
						gets (temp);
					} while (strlen (temp) > (DATE_PASSED_BUFFER - 1));
					strcpy (updateNode->data->date_passed, temp);
					do {
						printf ("Academic Year (yyyy-yy - %d characters): ", (AKADHMAIKO_ETOS_BUFFER - 1));
						gets (temp);
					} while (strlen (temp) > (AKADHMAIKO_ETOS_BUFFER - 1));
					strcpy (updateNode->data->akadhmaiko_etos, temp);
					do {
						printf ("Semester (integer): ");
						gets (temp);
					} while ((!numeric_int (temp)) || (strlen (temp) > (4 * REPLY_BUFFER - 1)));
					updateNode->data->semester = atoi (temp);
					do {
						printf ("Barythta (YP || BK || EP || PT): ");
						gets (temp);
					} while ((strlen (temp) > (BARYTHTA_BUFFER - 1)) || (!valid_barythta (temp)));
					strcpy (updateNode->data->barythta, temp);
					do {
						printf ("Section (integer): ");
						gets (temp);
					} while ((!numeric_int (temp)) || (strlen (temp) > (4 * REPLY_BUFFER - 1)));
					updateNode->data->section = atoi (temp);

					verify = data_record_verification (updateNode->data);
				} while (verify != 1);

				updateNode->key = updateNode->data->code;	// Provide node's key.
				searchNode = TREE__Search (Tree, updateNode->key);
				if ((searchNode == Tree->NIL) || (searchNode == current)) {
					/* 
					 * Update is allowed since
					 * no other node has the same key,
					 * or the record with the same key
					 * that was found is the node we 
					 * want to update.
					 */
					verify = TREE_NODE__Destroy (RED_BLACK__Delete (Tree, current));
					updated = RED_BLACK__Insert (Tree, updateNode);		// Indicate it was updated.
				}
				else {
					updated = 0;	// Indicate it was NOT updated.
				}
				if (verify && updated) {
					printf ("\nNode was successfully updated\n");
					printf ("\n");
					printf ("Press ENTER key to return to Update Menu...");
					fflush (stdout);
					fflush (stdin);
					gets (temp);
					break;
				}
				else {
					printf ("\nUpdate operation didn't take place.\n");
					printf ("A record was found with the same key!\n\a");
				}
			}
			else {
				printf ("\nNode is retained.\n");
			}
			printf ("\n");
			printf ("Press ENTER key to continue...");
			fflush (stdout);
			fflush (stdin);
			gets (temp);
		}
		if ((reply == 'n') || (reply == 'N')) {
			tempNode = TREE__Successor (Tree, current);
			successor = 1;
		}
		if ((reply == 'p') || (reply == 'P')) {
			tempNode = TREE__Predecessor (Tree, current);
			successor = -1;
		}
		if (tempNode != Tree->NIL)
			current = tempNode;
	} while ((reply != 'l') && (reply != 'L'));

	/*
	 * return 1 => Something was updated.
	 * return 0 => Nothing was updated.
	 */

	if (verified == 'y')
		return 1;
	else
		return 0;
}

char update_record_verification (TREEPTR Tree, TREE_NODEPTR node)
{
	char reply_string [REPLY_BUFFER];
	char reply;

	do {
		present_record (Tree, node);
		printf ("Are you sure you want to update this record?\n");
		printf ("(y = YES, n = NO) ? ");
		fflush (stdout);
		fflush (stdin);
		gets (reply_string);
		if (strlen (reply_string) != 1)
			reply = 'a';
		else
			reply = reply_string [0];
	} while ((reply != 'y') && (reply != 'n') && 
			(reply != 'Y') && (reply != 'N'));

	if ((reply == 'y') || (reply == 'Y'))
		return 'y';
	else
		return 'n';
}

/*********************************************************/

int key_update_operation (TREEPTR Tree, KEY_TYPE code)
{
	TREE_NODEPTR NodeReturned, updateNode, searchNode;
	char reply;
	char temp [4 * REPLY_BUFFER];
	int found;
	int updated;
	int verify;

	NodeReturned = TREE__Search (Tree, code);
	if (NodeReturned == Tree->NIL) {
		found = 0;		// Indicate it was not found.
		updated = 0;	// Indicated it was not updated.
		CLEARSCREEN;
		show_logo ();
		printf ("No valid record with the specific key could be found!\n\n\n");
		printf ("Press ENTER key to return to Update Menu...\a");
		fflush (stdout);
		fflush (stdin);
		gets (temp);
	}
	else {
		found = 1;		// Indicate it was found.
		do {
			present_record (Tree, NodeReturned);
			printf ("Do you want to update this record?\n");
			printf ("(y = YES, n = NO) ? ");
			fflush (stdout);
			fflush (stdin);
			gets (temp);
			if (strlen (temp) != 1)
				reply = 'a';
			else
				reply = temp [0];
		} while ((reply != 'y') && (reply != 'n') && 
				(reply != 'Y') && (reply != 'N'));

		if ((reply == 'y') || (reply == 'Y')) {
			updateNode = TREE_NODE__Create (Tree);
			do {
				CLEARSCREEN;
				show_logo ();
				printf ("Old Data Fields\n");
				printf ("---------------\n");
				printf ("NAME      : %s\n", NodeReturned->data->lesson);
				printf ("CODE      : %ld\n", NodeReturned->data->code);
				printf ("GRADE     : %.1f\n", NodeReturned->data->grade);
				printf ("DATE      : %s\n", NodeReturned->data->date_passed);
				printf ("BARYTHTA  : %s\n", NodeReturned->data->barythta);
				printf ("ACAD. YEAR: %s\n", NodeReturned->data->akadhmaiko_etos);
				printf ("SECTION   : %d\n", NodeReturned->data->section);
				printf ("SEMESTER  : %d\n", NodeReturned->data->semester);
				printf ("\n\n");
				printf ("Update Data Fileds\n");
				printf ("------------------\n");
				do {
					printf ("Lesson Code (long integer): ");
					gets (temp);
				} while ((!numeric_int (temp)) || (strlen (temp) > (4 * REPLY_BUFFER - 1)));
				updateNode->data->code = atol (temp);
				do {
					printf ("Lesson Name (max %d characters): ", (LESSON_NAME_BUFFER - 1));
					gets (temp);
				} while (strlen (temp) > (LESSON_NAME_BUFFER - 1));
				strcpy (updateNode->data->lesson, temp);
				do {
					printf ("Grade (double): ");
					gets (temp);
				} while (strlen (temp) > (4 * REPLY_BUFFER - 1));
				updateNode->data->grade = atof (temp);
				do {
					printf ("Date Passed (dd/mm/yyyy - %d characters): ", (DATE_PASSED_BUFFER - 1));
					gets (temp);
				} while (strlen (temp) > (DATE_PASSED_BUFFER - 1));
				strcpy (updateNode->data->date_passed, temp);
				do {
					printf ("Academic Year (yyyy-yy - %d characters): ", (AKADHMAIKO_ETOS_BUFFER - 1));
					gets (temp);
				} while (strlen (temp) > (AKADHMAIKO_ETOS_BUFFER - 1));
				strcpy (updateNode->data->akadhmaiko_etos, temp);
				do {
					printf ("Semester (integer): ");
					gets (temp);
				} while ((!numeric_int (temp)) || (strlen (temp) > (4 * REPLY_BUFFER - 1)));
				updateNode->data->semester = atoi (temp);
				do {
					printf ("Barythta (YP || BK || EP || PT): ");
					gets (temp);
				} while ((strlen (temp) > (BARYTHTA_BUFFER - 1)) || (!valid_barythta (temp)));
				strcpy (updateNode->data->barythta, temp);
				do {
					printf ("Section (integer): ");
					gets (temp);
				} while ((!numeric_int (temp)) || (strlen (temp) > (4 * REPLY_BUFFER - 1)));
				updateNode->data->section = atoi (temp);

				verify = data_record_verification (updateNode->data);
			} while (verify != 1);

			updateNode->key = updateNode->data->code;
			searchNode = TREE__Search (Tree, updateNode->key);
			if ((searchNode == Tree->NIL) || (searchNode == NodeReturned)) {
				/* 
				 * Update is allowed since
				 * no other node has the same key,
				 * or the record with the same key
				 * that was found is the node we 
				 * want to update.
				 */
				verify = TREE_NODE__Destroy (RED_BLACK__Delete (Tree, NodeReturned));
				updated = RED_BLACK__Insert (Tree, updateNode);		// Indicate it was updated.
			}
			else {
				updated = 0;
			}

			if (verify && updated)
				printf ("\nNode was successfully updated\n");
			else {
				printf ("\nUpdate operation didn't take place.\n");
				printf ("A record was found with the same key!\n\a");
			}
		}
		else {
			printf ("\nNode is retained.\n");
		}
		printf ("\n");
		printf ("Press ENTER key to return to Update Menu...");
		fflush (stdout);
		fflush (stdin);
		gets (temp);
	}

	if (!found)
		return 0;	// Indicate node with key: code  was not found.

	if (found && !updated)
		return -1;	// Indicate unsuccessful update operation. The node was found but not updated.

	return 1;		// Indicate successful update operation.
}


int search_operation (TREEPTR Tree, KEY_TYPE code)
{
	TREE_NODEPTR NodeReturned;
	char temp [REPLY_BUFFER];
	int found;

	NodeReturned = TREE__Search (Tree, code);
	if (NodeReturned == Tree->NIL) {
		CLEARSCREEN;
		show_logo ();
		printf ("No valid record with the specific key could be found!\n\n\n");
		printf ("Press ENTER key to return to Search Menu...\a");
		found = 0;		// Indicate that node was not found.
	}
	else {
		present_record (Tree, NodeReturned);
		printf ("Press ENTER key to return to Search Menu...");
		found = 1;		// Indicate that node was found.
	}

	fflush (stdout);
	fflush (stdin);
	gets (temp);

	return found;	// Indicate result of search operation.
}

/*********************************************************/

int period_lessons (TREEPTR Tree)
{
	char date_passed [DATE_PASSED_BUFFER];
	char *tokenPtr;
	char temp [DATE_PASSED_BUFFER];
	char barythta [BARYTHTA_BUFFER];
	char enter [REPLY_BUFFER];
	KEY_TYPE lesson_keys [ARRAY_SIZE];
	KEY_TYPE february [ARRAY_SIZE], june [ARRAY_SIZE], september [ARRAY_SIZE];
	KEY_TYPE key;
	TREE_NODEPTR current;
	int i, j, k, feb, jun, sep;
	int feb_kormou, feb_basika, feb_epiloghs, feb_ptyxiakh, 
		jun_kormou, jun_basika, jun_epiloghs, jun_ptyxiakh, 
		sep_kormou, sep_basika, sep_epiloghs, sep_ptyxiakh;
	int synolo;
	int years [ARRAY_SIZE], hmeromhnia [ARRAY_SIZE];
	int xronos;

	for (i = 1; i <= ARRAY_SIZE; i++) {
		years [i - 1] = 0;
	}

	current = TREE__Minimum (Tree, Tree->ROOT);
	while (current != Tree->NIL) {
		strcpy (date_passed, current->data->date_passed);
		tokenPtr = strtok (date_passed, "/");
		for (i = 1; i <= 2; i++) {
			tokenPtr = strtok (NULL, "/");
		}
		strcpy (temp, tokenPtr);
		xronos = atoi (temp);
		insert_year (xronos, years);
		/* Examine next Tree node. */
		current = TREE__Successor (Tree, current);
	}

	i = 0;
	CLEARSCREEN;
	show_logo ();
	printf ("                     NUMBER OF LESSONS PASSED\n");
	printf ("                     ------------------------\n");
	printf ("EXAMINATION PERIOD              1    1    2  \tKIND OF LESSONS PASSED\n");
	printf ("------------------    0    5    0    5    0  \t----------------------\n");
	fflush (stdout);
	while (years [i] != 0) {
		for (k = 1; k <= ARRAY_SIZE; k++) {
			hmeromhnia [k - 1] = 1300;		// Impossible to be a correct day.
		}
		find_lesson_keys (years [i], Tree, lesson_keys);	//Find lessons passed this year.
		j = 0;
		while (lesson_keys [j] != 0) {
			key  = lesson_keys [j];
			current = TREE__Search (Tree, key);
			strcpy (date_passed, current->data->date_passed);
			tokenPtr = strtok (date_passed, "/");
			for (k = 1; k <= 2; k++) {
				strcpy (temp, tokenPtr);
				if (k == 1)
					hmeromhnia [j] = atoi (temp);
				else
					hmeromhnia [j] += 100 * atoi (temp);
				tokenPtr = strtok (NULL, "/");
			}
			j++;
		}
		QuickSort_int (hmeromhnia, 0, ARRAY_SIZE - 1, lesson_keys);
		/* Now lesson_keys are sorted in increasing days of year. */

		j = 0;
		for (k = 1; k <= ARRAY_SIZE; k++) {
			february [k - 1] = 0;
			june [k - 1] = 0;
			september [k - 1] = 0;
		}
		feb = 0;
		jun = 0;
		sep = 0;
		feb_kormou = 0;
		feb_basika = 0;
		feb_epiloghs = 0;
		feb_ptyxiakh = 0;
		jun_kormou = 0;
		jun_basika = 0;
		jun_epiloghs = 0;
		jun_ptyxiakh = 0;
		sep_kormou = 0;
		sep_basika = 0;
		sep_epiloghs = 0;
		sep_ptyxiakh = 0;
		while (lesson_keys [j] != 0) {
			key = lesson_keys [j];
			current = TREE__Search (Tree, key);
			strcpy (barythta, current->data->barythta);
			if ((hmeromhnia [j] > 100) && (hmeromhnia [j] < 400)) {
				february [feb] = lesson_keys [j];
				if (strcmp (barythta, "YP") == 0)
					feb_kormou++;
				if (strcmp (barythta, "BK") == 0)
					feb_basika++;
				if (strcmp (barythta, "EP") == 0)
					feb_epiloghs++;
				if (strcmp (barythta, "PT") == 0)
					feb_ptyxiakh++;
				feb++;
			}
			if ((hmeromhnia [j] > 500) && (hmeromhnia [j] < 800)) {
				june [jun] = lesson_keys [j];
				if (strcmp (barythta, "YP") == 0)
					jun_kormou++;
				if (strcmp (barythta, "BK") == 0)
					jun_basika++;
				if (strcmp (barythta, "EP") == 0)
					jun_epiloghs++;
				if (strcmp (barythta, "PT") == 0)
					jun_ptyxiakh++;
				jun++;
			}
			if ((hmeromhnia [j] > 800) && (hmeromhnia [j] < 1100)) {
				september [sep] = lesson_keys [j];
				if (strcmp (barythta, "YP") == 0)
					sep_kormou++;
				if (strcmp (barythta, "BK") == 0)
					sep_basika++;
				if (strcmp (barythta, "EP") == 0)
					sep_epiloghs++;
				if (strcmp (barythta, "PT") == 0)
					sep_ptyxiakh++;
				sep++;
			}
			j++;
		}

		printf ("     February %-4d    |", years [i]);
		for (k = 1; k <= 22; k++) {
			if (k <= feb)
				printf ("*");
			else
				printf (" ");
		}
		printf ("\t");
		synolo = feb_kormou + feb_basika + feb_epiloghs + feb_ptyxiakh;
		if (synolo > 0) {
			if ((feb_kormou < synolo) && (feb_kormou > 0))
				printf ("YP=%d, ", feb_kormou);
			else {
				if (feb_kormou)
					printf ("YP=%d", feb_kormou);
			}
			if ((feb_kormou + feb_basika < synolo) && (feb_basika > 0))
				printf ("BK=%d, ", feb_basika);
			else {
				if (feb_basika)
					printf ("BK=%d", feb_basika);
			}
			if ((feb_kormou + feb_basika + feb_epiloghs < synolo) && (feb_epiloghs > 0))
				printf ("EP=%d, ", feb_epiloghs);
			else {
				if (feb_epiloghs)
					printf ("EP=%d", feb_epiloghs);
			}
			if (feb_ptyxiakh)
				printf ("PTYXIAKH");
		}
		printf ("\n");

		/* H if yparxei gia to endexomeno h eksetastikh Febrouariou toy trexontos */
		/* etous na einai kai h teleytaia eksetastikh gia ton sygkekrimeno foithth */
		if ( ((jun > 0) || (sep > 0)) || (years [i + 1] != 0)) {
			printf ("         June %-4d    |", years [i]);
			for (k = 1; k <= 22; k++) {
				if (k <= jun)
					printf ("*");
				else
					printf (" ");
			}
			printf ("\t");
			synolo = jun_kormou + jun_basika + jun_epiloghs + jun_ptyxiakh;
			if (synolo > 0) {
				if ((jun_kormou < synolo) && (jun_kormou > 0))
					printf ("YP=%d, ", jun_kormou);
				else {
					if (jun_kormou)
						printf ("YP=%d", jun_kormou);
				}
				if ((jun_kormou + jun_basika < synolo) && (jun_basika > 0))
					printf ("BK=%d, ", jun_basika);
				else {
					if (jun_basika)
						printf ("BK=%d", jun_basika);
				}
				if ((jun_kormou + jun_basika + jun_epiloghs < synolo) && (jun_epiloghs > 0))
					printf ("EP=%d, ", jun_epiloghs);
				else {
					if (jun_epiloghs)
						printf ("EP=%d", jun_epiloghs);
				}
				if (jun_ptyxiakh)
					printf ("PTYXIAKH");
			}
			printf ("\n");
		}

		if ( (sep > 0) || (years [i + 1] != 0)) {
			printf ("    September %-4d    |", years [i]);
			for (k = 1; k <= 22; k++) {
				if (k <= sep)
					printf ("*");
				else
					printf (" ");
			}
			printf ("\t");
			synolo = sep_kormou + sep_basika + sep_epiloghs + sep_ptyxiakh;
			if (synolo > 0) {
				if ((sep_kormou < synolo) && (sep_kormou > 0))
					printf ("YP=%d, ", sep_kormou);
				else {
					if (sep_kormou)
						printf ("YP=%d", sep_kormou);
				}
				if ((sep_kormou + sep_basika < synolo) && (sep_basika > 0))
					printf ("BK=%d, ", sep_basika);
				else {
					if (sep_basika)
						printf ("BK=%d", sep_basika);
				}
				if ((sep_kormou + sep_basika + sep_epiloghs < synolo) && (sep_epiloghs > 0))
					printf ("EP=%d, ", sep_epiloghs);
				else {
					if (sep_epiloghs)
						printf ("EP=%d", sep_epiloghs);
				}
				if (sep_ptyxiakh)
					printf ("PTYXIAKH");
			}
			printf ("\n");
		}

		/* Go to next year */
		i++;
	}


	printf ("\n\n");
	printf ("Press ENTER key to return to Statistics Menu...");
	fflush (stdout);
	fflush (stdin);
	gets (enter);

	return 1;	// Indicate succesful finish.
}

int period_average (TREEPTR Tree)
{
	char date_passed [DATE_PASSED_BUFFER];
	char *tokenPtr;
	char temp [DATE_PASSED_BUFFER];
	char barythta [BARYTHTA_BUFFER];
	char enter [REPLY_BUFFER];
	KEY_TYPE lesson_keys [ARRAY_SIZE];
	KEY_TYPE february [ARRAY_SIZE], june [ARRAY_SIZE], september [ARRAY_SIZE];
	KEY_TYPE key;
	TREE_NODEPTR current;
	int i, j, k, l, feb, jun, sep;
	double feb_avg, jun_avg, sep_avg;
	int years [ARRAY_SIZE], hmeromhnia [ARRAY_SIZE];
	int xronos;

	for (i = 1; i <= ARRAY_SIZE; i++) {
		years [i - 1] = 0;
	}

	current = TREE__Minimum (Tree, Tree->ROOT);
	while (current != Tree->NIL) {
		strcpy (date_passed, current->data->date_passed);
		tokenPtr = strtok (date_passed, "/");
		for (i = 1; i <= 2; i++) {
			tokenPtr = strtok (NULL, "/");
		}
		strcpy (temp, tokenPtr);
		xronos = atoi (temp);
		insert_year (xronos, years);
		/* Examine next Tree node. */
		current = TREE__Successor (Tree, current);
	}

	i = 0;
	CLEARSCREEN;
	show_logo ();
	printf ("                               AVERAGE GRADE OF LESSONS PASSED\n");
	printf ("                               -------------------------------\n");
	printf ("EXAMINATION PERIOD                                                     \n");
	printf ("------------------   5....|....6....|....7....|....8....|....9....|....10\n");
	fflush (stdout);
	while (years [i] != 0) {
		for (k = 1; k <= ARRAY_SIZE; k++) {
			hmeromhnia [k - 1] = 1300;		//Impossible to be correct.
		}
		find_lesson_keys (years [i], Tree, lesson_keys);	//Find lessons passed this year.

		j = 0;
		while (lesson_keys [j] != 0) {
			key  = lesson_keys [j];
			current = TREE__Search (Tree, key);
			strcpy (date_passed, current->data->date_passed);
			tokenPtr = strtok (date_passed, "/");
			for (k = 1; k <= 2; k++) {
				strcpy (temp, tokenPtr);
				if (k == 1)
					hmeromhnia [j] = atoi (temp);
				else
					hmeromhnia [j] += 100 * atoi (temp);
				tokenPtr = strtok (NULL, "/");
			}
			j++;
		}
		QuickSort_int (hmeromhnia, 0, ARRAY_SIZE - 1, lesson_keys);
		/* Now lesson_keys are sorted in increasing days of year. */

		j = 0;
		for (k = 1; k <= ARRAY_SIZE; k++) {
			february [k - 1] = 0;
			june [k - 1] = 0;
			september [k - 1] = 0;
		}
		feb = 0;
		jun = 0;
		sep = 0;
		while (lesson_keys [j] != 0) {
			key = lesson_keys [j];
			current = TREE__Search (Tree, key);
			strcpy (barythta, current->data->barythta);
			if ((hmeromhnia [j] > 100) && (hmeromhnia [j] < 400)) {
				february [feb] = lesson_keys [j];
				feb++;
			}
			if ((hmeromhnia [j] > 500) && (hmeromhnia [j] < 800)) {
				june [jun] = lesson_keys [j];
				jun++;
			}
			if ((hmeromhnia [j] > 800) && (hmeromhnia [j] < 1100)) {
				september [sep] = lesson_keys [j];
				sep++;
			}
			j++;
		}

		feb_avg = calculate_period_average (Tree, february);
		jun_avg = calculate_period_average (Tree, june);
		sep_avg = calculate_period_average (Tree, september);

		if (feb < 10)
			printf ("(%d)  February %-4d  |", feb, years [i]);
		else
			printf ("(%2d) February %-4d  |", feb, years [i]);
		l = stroggylopoihsh (feb_avg * 10);
		l -= 49;
		for (k = 1; k <= l; k++) {
			printf ("*");
		}
		printf ("\n");

		/* H if yparxei gia to endexomeno h eksetastikh Febrouariou toy trexontos */
		/* etous na einai kai h teleytaia eksetastikh gia ton sygkekrimeno foithth */
		if ( ((jun > 0) || (sep > 0)) || (years [i + 1] != 0)) {
			if (jun < 10)
				printf ("(%d)      June %-4d  |", jun, years [i]);
			else
				printf ("(%2d)     June %-4d  |", jun, years [i]);
			l = stroggylopoihsh (jun_avg * 10);
			l -= 49;
			for (k = 1; k <= l; k++) {
				printf ("*");
			}
			printf ("\n");
		}

		if ( (sep > 0) || (years [i + 1] != 0)) {
			if (sep < 10)
				printf ("(%d) September %-4d  |", sep, years [i]);
			else
				printf ("(%2d)September %-4d  |", sep, years [i]);
			l = stroggylopoihsh (sep_avg * 10);
			l -= 49;
			for (k = 1; k <= l; k++) {
				printf ("*");
			}
			printf ("\n");
		}

		/* Go to next year */
		i++;
	}


	printf ("\n\n");
	printf ("Press ENTER key to return to Statistics Menu...");
	fflush (stdout);
	fflush (stdin);
	gets (enter);

	return 1;	// Indicate succesful finish.
}

int average_fluctuation (TREEPTR Tree, const int tomeas)
{
	char date_passed [DATE_PASSED_BUFFER];
	char *tokenPtr;
	char temp [DATE_PASSED_BUFFER];
	char barythta [BARYTHTA_BUFFER];
	char enter [REPLY_BUFFER];
	KEY_TYPE lesson_keys [ARRAY_SIZE];
	LISTPTR february, june, september;
	KEY_TYPE key;
	TREE_NODEPTR current;
	int i, j, k, l, feb, jun, sep;
	double feb_avg, jun_avg, sep_avg;
	int years [ARRAY_SIZE], hmeromhnia [ARRAY_SIZE];
	int xronos;
	FLUCTUATION_AVERAGEPTR average = (FLUCTUATION_AVERAGEPTR) malloc (sizeof (FLUCTUATION_AVERAGE));
	average->ypoxrewtika = LIST__Initialize ();
	average->basika = LIST__Initialize ();
	average->epiloghs = LIST__Initialize ();

	average->number_of_lessons = 0;
	average->arithmiths = 0.0;
	average->paronomasths = 0.0;
	average->average_grade = 0.0;

	for (i = 1; i <= ARRAY_SIZE; i++) {
		years [i - 1] = 0;
	}

	current = TREE__Minimum (Tree, Tree->ROOT);
	while (current != Tree->NIL) {
		strcpy (date_passed, current->data->date_passed);
		tokenPtr = strtok (date_passed, "/");
		for (i = 1; i <= 2; i++) {
			tokenPtr = strtok (NULL, "/");
		}
		strcpy (temp, tokenPtr);
		xronos = atoi (temp);
		insert_year (xronos, years);
		/* Examine next Tree node. */
		current = TREE__Successor (Tree, current);
	}

	i = 0;
	CLEARSCREEN;
	show_logo ();
	if (tomeas == 0) {
		printf ("                            UNI-SECTION AVERAGE GRADE FLUCTUATION\n");
		printf ("                            -------------------------------------\n");
	}
	else {
		switch (tomeas) {
			case 1:
				printf ("                       AVERAGE GRADE FLUCTUATION FOLLOWING 1st SECTION\n");
				printf ("                       -----------------------------------------------\n");
				break;
			case 2:
				printf ("                       AVERAGE GRADE FLUCTUATION FOLLOWING 2nd SECTION\n");
				printf ("                       -----------------------------------------------\n");
				break;
			case 3:
				printf ("                       AVERAGE GRADE FLUCTUATION FOLLOWING 3rd SECTION\n");
				printf ("                       -----------------------------------------------\n");
				break;
		}
	}
	printf ("EXAMINATION PERIOD                                                     \n");
	printf ("------------------   5....|....6....|....7....|....8....|....9....|....10\n");
	fflush (stdout);
	while (years [i] != 0) {
		for (k = 1; k <= ARRAY_SIZE; k++) {
			hmeromhnia [k - 1] = 1300;		//Impossible to be correct.
		}
		find_lesson_keys (years [i], Tree, lesson_keys);	//Find lessons passed this year.

		j = 0;
		while (lesson_keys [j] != 0) {
			key  = lesson_keys [j];
			current = TREE__Search (Tree, key);
			strcpy (date_passed, current->data->date_passed);
			tokenPtr = strtok (date_passed, "/");
			for (k = 1; k <= 2; k++) {
				strcpy (temp, tokenPtr);
				if (k == 1)
					hmeromhnia [j] = atoi (temp);
				else
					hmeromhnia [j] += 100 * atoi (temp);
				tokenPtr = strtok (NULL, "/");
			}
			j++;
		}
		QuickSort_int (hmeromhnia, 0, ARRAY_SIZE - 1, lesson_keys);
		/* Now lesson_keys are sorted in increasing days of year. */

		february = LIST__Initialize ();
		june = LIST__Initialize ();
		september = LIST__Initialize ();
		j = 0;
		feb = 0;
		jun = 0;
		sep = 0;
		while (lesson_keys [j] != 0) {
			key = lesson_keys [j];
			current = TREE__Search (Tree, key);
			strcpy (barythta, current->data->barythta);
			if ((hmeromhnia [j] > 100) && (hmeromhnia [j] < 400)) {
				LIST__Insert_node_at_front (february, lesson_keys [j], current);
				feb++;
			}
			if ((hmeromhnia [j] > 500) && (hmeromhnia [j] < 800)) {
				LIST__Insert_node_at_front (june, lesson_keys [j], current);
				jun++;
			}
			if ((hmeromhnia [j] > 800) && (hmeromhnia [j] < 1100)) {
				LIST__Insert_node_at_front (september, lesson_keys [j], current);
				sep++;
			}
			j++;
		}

		manage_average_fluctuation (Tree, february, average, tomeas);
		feb_avg = average->average_grade;

		if (feb < 10)
			printf ("(%d)  February %-4d  |", feb, years [i]);
		else
			printf ("(%2d) February %-4d  |", feb, years [i]);
		l = stroggylopoihsh (feb_avg * 10);
		l -= 49;
		for (k = 1; k <= l; k++) {
			printf ("*");
		}
		printf ("\n");

		/* H if yparxei gia to endexomeno h eksetastikh Febrouariou toy trexontos */
		/* etous na einai kai h teleytaia eksetastikh gia ton sygkekrimeno foithth */
		if ( ((jun > 0) || (sep > 0)) || (years [i + 1] != 0)) {
			manage_average_fluctuation (Tree, june, average, tomeas);
			jun_avg = average->average_grade;
			if (jun < 10)
				printf ("(%d)      June %-4d  |", jun, years [i]);
			else
				printf ("(%2d)     June %-4d  |", jun, years [i]);
			l = stroggylopoihsh (jun_avg * 10);
			l -= 49;
			for (k = 1; k <= l; k++) {
				printf ("*");
			}
			printf ("\n");
		}

		if ( (sep > 0) || (years [i + 1] != 0)) {
			manage_average_fluctuation (Tree, september, average, tomeas);
			sep_avg = average->average_grade;
			if (sep < 10)
				printf ("(%d) September %-4d  |", sep, years [i]);
			else
				printf ("(%2d)September %-4d  |", sep, years [i]);
			l = stroggylopoihsh (sep_avg * 10);
			l -= 49;
			for (k = 1; k <= l; k++) {
				printf ("*");
			}
			printf ("\n");
		}

		/* Dealloccate useless memory */
		LIST__Destroy (february);
		LIST__Destroy (june);
		LIST__Destroy (september);

		/* Go to next year */
		i++;
	}

	/* Deallocate memory */
	LIST__Destroy (average->ypoxrewtika);
	LIST__Destroy (average->basika);
	LIST__Destroy (average->epiloghs);
	free (average);

	printf ("\n\n");
	printf ("Press ENTER key to return to Average Grade Fluctuation Menu...");
	fflush (stdout);
	fflush (stdin);
	gets (enter);

	return 1;	// Indicate succesful finish.
}

int lesson_grades_stats (TREEPTR Tree, OWNERPTR owner, FILE *somewhere, char *barythta)
{
	int i, plhthos;
	double grade;

	fprintf (somewhere, "***********************************************\n");
	fprintf (somewhere, "* UNIVERSITY OF ATHENS                        *\n");
	fprintf (somewhere, "* DEPT. OF INFORMATICS AND TELECOMMUNICATIONS *\n");
	fprintf (somewhere, "***********************************************\n");
	fprintf (somewhere, "\n\n");
	if (somewhere != stdout) {
		fprintf (somewhere, "STUDENT CHARACTERISTICS\n");
		fprintf (somewhere, "-----------------------\n");
		fprintf (somewhere, "First Name: %s\n", owner->firstname);
		fprintf (somewhere, "Last Name : %s\n", owner->lastname);
		fprintf (somewhere, "Id. Number: %s\n", owner->id);
		fprintf (somewhere, "Entry Date: %s\n", owner->entry_date);
		fprintf (somewhere, "Birthday  : %s\n", owner->birthday);
		fprintf (somewhere, "\n\n");
	}
	if (strcmp (barythta, "YP") == 0) {
		fprintf (somewhere, "         NUMBER OF  \"YP\" LESSONS - ALL SECTIONS\n");
		fprintf (somewhere, "         --------------------------------------\n");
		fprintf (somewhere, "GRADE            1    1    2    2    3    3    4\n");
		fprintf (somewhere, "-----   1...5....0....5....0....5....0....5....0\n");
		fflush (somewhere);
		grade = 5.0;
		while (grade <= 10.0) {
			plhthos = number_of_lessons_with_grade_and_certain_barythta ( Tree, grade, "YP");
			if ((grade - floor (grade) == 0.5) && (plhthos == 0)) {
				grade += 0.5;
				continue;		// Do not show special cases if not necessary!
			}
			fprintf (somewhere, "%4.1f:   ", grade);;
			for (i = 1; i <= plhthos; i++)
				fprintf (somewhere, "*");
			fprintf (somewhere, "\n");
			fflush (somewhere);
			grade += 0.5;
		}
		fprintf (somewhere, "\n");

		fprintf (somewhere, "          NUMBER OF \"YP\" LESSONS - 1st SECTION\n");
		fprintf (somewhere, "          ------------------------------------\n");
		fprintf (somewhere, "GRADE            1    1    2    2    3    3    4\n");
		fprintf (somewhere, "-----   1...5....0....5....0....5....0....5....0\n");
		fflush (somewhere);
		grade = 5.0;
		while (grade <= 10.0) {
			plhthos = number_of_lessons_with_grade_certain_barythta_and_section ( Tree, grade, "YP", 1);
			if ((grade - floor (grade) == 0.5) && (plhthos == 0)) {
				grade += 0.5;
				continue;		// Do not show special cases if not necessary!
			}
			fprintf (somewhere, "%4.1f:   ", grade);;
			for (i = 1; i <= plhthos; i++)
				fprintf (somewhere, "*");
			fprintf (somewhere, "\n");
			fflush (somewhere);
			grade += 0.5;
		}
		fprintf (somewhere, "\n");

		fprintf (somewhere, "          NUMBER OF \"YP\" LESSONS - 2nd SECTION\n");
		fprintf (somewhere, "          ------------------------------------\n");
		fprintf (somewhere, "GRADE            1    1    2    2    3    3    4\n");
		fprintf (somewhere, "-----   1...5....0....5....0....5....0....5....0\n");
		fflush (somewhere);
		grade = 5.0;
		while (grade <= 10.0) {
			plhthos = number_of_lessons_with_grade_certain_barythta_and_section ( Tree, grade, "YP", 2);
			if ((grade - floor (grade) == 0.5) && (plhthos == 0)) {
				grade += 0.5;
				continue;		// Do not show special cases if not necessary!
			}
			fprintf (somewhere, "%4.1f:   ", grade);;
			for (i = 1; i <= plhthos; i++)
				fprintf (somewhere, "*");
			fprintf (somewhere, "\n");
			fflush (somewhere);
			grade += 0.5;
		}
		fprintf (somewhere, "\n");

		fprintf (somewhere, "          NUMBER OF \"YP\" LESSONS - 3rd SECTION\n");
		fprintf (somewhere, "          ------------------------------------\n");
		fprintf (somewhere, "GRADE            1    1    2    2    3    3    4\n");
		fprintf (somewhere, "-----   1...5....0....5....0....5....0....5....0\n");
		fflush (somewhere);
		grade = 5.0;
		while (grade <= 10.0) {
			plhthos = number_of_lessons_with_grade_certain_barythta_and_section ( Tree, grade, "YP", 3);
			if ((grade - floor (grade) == 0.5) && (plhthos == 0)) {
				grade += 0.5;
				continue;		// Do not show special cases if not necessary!
			}
			fprintf (somewhere, "%4.1f:   ", grade);;
			for (i = 1; i <= plhthos; i++)
				fprintf (somewhere, "*");
			fprintf (somewhere, "\n");
			fflush (somewhere);
			grade += 0.5;
		}
	}
	else if (strcmp (barythta, "EP") == 0) {
		fprintf (somewhere, "         NUMBER OF  \"EP\" LESSONS - ALL SECTIONS\n");
		fprintf (somewhere, "         --------------------------------------\n");
		fprintf (somewhere, "GRADE            1    1    2    2    3    3    4\n");
		fprintf (somewhere, "-----   1...5....0....5....0....5....0....5....0\n");
		fflush (somewhere);
		grade = 5.0;
		while (grade <= 10.0) {
			plhthos = number_of_lessons_with_grade_and_certain_barythta ( Tree, grade, "EP");
			if ((grade - floor (grade) == 0.5) && (plhthos == 0)) {
				grade += 0.5;
				continue;		// Do not show special cases if not necessary!
			}
			fprintf (somewhere, "%4.1f:   ", grade);;
			for (i = 1; i <= plhthos; i++)
				fprintf (somewhere, "*");
			fprintf (somewhere, "\n");
			fflush (somewhere);
			grade += 0.5;
		}
		fprintf (somewhere, "\n");

		fprintf (somewhere, "          NUMBER OF \"EP\" LESSONS - 1st SECTION\n");
		fprintf (somewhere, "          ------------------------------------\n");
		fprintf (somewhere, "GRADE            1    1    2    2    3    3    4\n");
		fprintf (somewhere, "-----   1...5....0....5....0....5....0....5....0\n");
		fflush (somewhere);
		grade = 5.0;
		while (grade <= 10.0) {
			plhthos = number_of_lessons_with_grade_certain_barythta_and_section ( Tree, grade, "EP", 1);
			if ((grade - floor (grade) == 0.5) && (plhthos == 0)) {
				grade += 0.5;
				continue;		// Do not show special cases if not necessary!
			}
			fprintf (somewhere, "%4.1f:   ", grade);;
			for (i = 1; i <= plhthos; i++)
				fprintf (somewhere, "*");
			fprintf (somewhere, "\n");
			fflush (somewhere);
			grade += 0.5;
		}
		fprintf (somewhere, "\n");

		fprintf (somewhere, "          NUMBER OF \"EP\" LESSONS - 2nd SECTION\n");
		fprintf (somewhere, "          ------------------------------------\n");
		fprintf (somewhere, "GRADE            1    1    2    2    3    3    4\n");
		fprintf (somewhere, "-----   1...5....0....5....0....5....0....5....0\n");
		fflush (somewhere);
		grade = 5.0;
		while (grade <= 10.0) {
			plhthos = number_of_lessons_with_grade_certain_barythta_and_section ( Tree, grade, "EP", 2);
			if ((grade - floor (grade) == 0.5) && (plhthos == 0)) {
				grade += 0.5;
				continue;		// Do not show special cases if not necessary!
			}
			fprintf (somewhere, "%4.1f:   ", grade);;
			for (i = 1; i <= plhthos; i++)
				fprintf (somewhere, "*");
			fprintf (somewhere, "\n");
			fflush (somewhere);
			grade += 0.5;
		}
		fprintf (somewhere, "\n");

		fprintf (somewhere, "          NUMBER OF \"EP\" LESSONS - 3rd SECTION\n");
		fprintf (somewhere, "          ------------------------------------\n");
		fprintf (somewhere, "GRADE            1    1    2    2    3    3    4\n");
		fprintf (somewhere, "-----   1...5....0....5....0....5....0....5....0\n");
		fflush (somewhere);
		grade = 5.0;
		while (grade <= 10.0) {
			plhthos = number_of_lessons_with_grade_certain_barythta_and_section ( Tree, grade, "EP", 3);
			if ((grade - floor (grade) == 0.5) && (plhthos == 0)) {
				grade += 0.5;
				continue;		// Do not show special cases if not necessary!
			}
			fprintf (somewhere, "%4.1f:   ", grade);;
			for (i = 1; i <= plhthos; i++)
				fprintf (somewhere, "*");
			fprintf (somewhere, "\n");
			fflush (somewhere);
			grade += 0.5;
		}
	}
	else if (strcmp (barythta, "BK") == 0) {
		fprintf (somewhere, "         NUMBER OF  \"BK\" LESSONS - ALL SECTIONS\n");
		fprintf (somewhere, "         --------------------------------------\n");
		fprintf (somewhere, "GRADE            1    1    2    2    3    3    4\n");
		fprintf (somewhere, "-----   1...5....0....5....0....5....0....5....0\n");
		fflush (somewhere);
		grade = 5.0;
		while (grade <= 10.0) {
			plhthos = number_of_lessons_with_grade_and_certain_barythta ( Tree, grade, "BK");
			if ((grade - floor (grade) == 0.5) && (plhthos == 0)) {
				grade += 0.5;
				continue;		// Do not show special cases if not necessary!
			}
			fprintf (somewhere, "%4.1f:   ", grade);;
			for (i = 1; i <= plhthos; i++)
				fprintf (somewhere, "*");
			fprintf (somewhere, "\n");
			fflush (somewhere);
			grade += 0.5;
		}
		fprintf (somewhere, "\n");

		fprintf (somewhere, "          NUMBER OF \"BK\" LESSONS - 1st SECTION\n");
		fprintf (somewhere, "          ------------------------------------\n");
		fprintf (somewhere, "GRADE            1    1    2    2    3    3    4\n");
		fprintf (somewhere, "-----   1...5....0....5....0....5....0....5....0\n");
		fflush (somewhere);
		grade = 5.0;
		while (grade <= 10.0) {
			plhthos = number_of_lessons_with_grade_certain_barythta_and_section ( Tree, grade, "BK", 1);
			if ((grade - floor (grade) == 0.5) && (plhthos == 0)) {
				grade += 0.5;
				continue;		// Do not show special cases if not necessary!
			}
			fprintf (somewhere, "%4.1f:   ", grade);;
			for (i = 1; i <= plhthos; i++)
				fprintf (somewhere, "*");
			fprintf (somewhere, "\n");
			fflush (somewhere);
			grade += 0.5;
		}
		fprintf (somewhere, "\n");

		fprintf (somewhere, "          NUMBER OF \"BK\" LESSONS - 2nd SECTION\n");
		fprintf (somewhere, "          ------------------------------------\n");
		fprintf (somewhere, "GRADE            1    1    2    2    3    3    4\n");
		fprintf (somewhere, "-----   1...5....0....5....0....5....0....5....0\n");
		fflush (somewhere);
		grade = 5.0;
		while (grade <= 10.0) {
			plhthos = number_of_lessons_with_grade_certain_barythta_and_section ( Tree, grade, "BK", 2);
			if ((grade - floor (grade) == 0.5) && (plhthos == 0)) {
				grade += 0.5;
				continue;		// Do not show special cases if not necessary!
			}
			fprintf (somewhere, "%4.1f:   ", grade);;
			for (i = 1; i <= plhthos; i++)
				fprintf (somewhere, "*");
			fprintf (somewhere, "\n");
			fflush (somewhere);
			grade += 0.5;
		}
		fprintf (somewhere, "\n");

		fprintf (somewhere, "          NUMBER OF \"BK\" LESSONS - 3rd SECTION\n");
		fprintf (somewhere, "          ------------------------------------\n");
		fprintf (somewhere, "GRADE            1    1    2    2    3    3    4\n");
		fprintf (somewhere, "-----   1...5....0....5....0....5....0....5....0\n");
		fflush (somewhere);
		grade = 5.0;
		while (grade <= 10.0) {
			plhthos = number_of_lessons_with_grade_certain_barythta_and_section ( Tree, grade, "BK", 3);
			if ((grade - floor (grade) == 0.5) && (plhthos == 0)) {
				grade += 0.5;
				continue;		// Do not show special cases if not necessary!
			}
			fprintf (somewhere, "%4.1f:   ", grade);;
			for (i = 1; i <= plhthos; i++)
				fprintf (somewhere, "*");
			fprintf (somewhere, "\n");
			fflush (somewhere);
			grade += 0.5;
		}
	}
	else if (strcmp (barythta, "ALL") == 0) {
		fprintf (somewhere, "              NUMBER OF LESSONS - OVERALL\n");
		fprintf (somewhere, "              ---------------------------\n");
		fprintf (somewhere, "GRADE            1    1    2    2    3    3    4\n");
		fprintf (somewhere, "-----   1...5....0....5....0....5....0....5....0\n");
		fflush (somewhere);
		grade = 5.0;
		while (grade <= 10.0) {
			plhthos = number_of_lessons_with_grade ( Tree, grade);
			if ((grade - floor (grade) == 0.5) && (plhthos == 0)) {
				grade += 0.5;
				continue;		// Do not show special cases if not necessary!
			}
			fprintf (somewhere, "%4.1f:   ", grade);;
			for (i = 1; i <= plhthos; i++)
				fprintf (somewhere, "*");
			fprintf (somewhere, "\n");
			fflush (somewhere);
			grade += 0.5;
		}
		fprintf (somewhere, "\n");

		fprintf (somewhere, "            NUMBER OF LESSONS - 1st SECTION\n");
		fprintf (somewhere, "            -------------------------------\n");
		fprintf (somewhere, "GRADE            1    1    2    2    3    3    4\n");
		fprintf (somewhere, "-----   1...5....0....5....0....5....0....5....0\n");
		fflush (somewhere);
		grade = 5.0;
		while (grade <= 10.0) {
			plhthos = number_of_lessons_with_grade_and_section ( Tree, grade, 1);
			if ((grade - floor (grade) == 0.5) && (plhthos == 0)) {
				grade += 0.5;
				continue;		// Do not show special cases if not necessary!
			}
			fprintf (somewhere, "%4.1f:   ", grade);;
			for (i = 1; i <= plhthos; i++)
				fprintf (somewhere, "*");
			fprintf (somewhere, "\n");
			fflush (somewhere);
			grade += 0.5;
		}
		fprintf (somewhere, "\n");

		fprintf (somewhere, "            NUMBER OF LESSONS - 2nd SECTION\n");
		fprintf (somewhere, "            -------------------------------\n");
		fprintf (somewhere, "GRADE            1    1    2    2    3    3    4\n");
		fprintf (somewhere, "-----   1...5....0....5....0....5....0....5....0\n");
		fflush (somewhere);
		grade = 5.0;
		while (grade <= 10.0) {
			plhthos = number_of_lessons_with_grade_and_section ( Tree, grade, 2);
			if ((grade - floor (grade) == 0.5) && (plhthos == 0)) {
				grade += 0.5;
				continue;		// Do not show special cases if not necessary!
			}
			fprintf (somewhere, "%4.1f:   ", grade);;
			for (i = 1; i <= plhthos; i++)
				fprintf (somewhere, "*");
			fprintf (somewhere, "\n");
			fflush (somewhere);
			grade += 0.5;
		}
		fprintf (somewhere, "\n");

		fprintf (somewhere, "            NUMBER OF LESSONS - 3rd SECTION\n");
		fprintf (somewhere, "            -------------------------------\n");
		fprintf (somewhere, "GRADE            1    1    2    2    3    3    4\n");
		fprintf (somewhere, "-----   1...5....0....5....0....5....0....5....0\n");
		fflush (somewhere);
		grade = 5.0;
		while (grade <= 10.0) {
			plhthos = number_of_lessons_with_grade_and_section ( Tree, grade, 3);
			if ((grade - floor (grade) == 0.5) && (plhthos == 0)) {
				grade += 0.5;
				continue;		// Do not show special cases if not necessary!
			}
			fprintf (somewhere, "%4.1f:   ", grade);;
			for (i = 1; i <= plhthos; i++)
				fprintf (somewhere, "*");
			fprintf (somewhere, "\n");
			fflush (somewhere);
			grade += 0.5;
		}
		fprintf (somewhere, "\n");

		fprintf (somewhere, "        NUMBER OF LESSONS - NO PARTICULAR SECTION\n");
		fprintf (somewhere, "        -----------------------------------------\n");
		fprintf (somewhere, "GRADE            1    1    2    2    3    3    4\n");
		fprintf (somewhere, "-----   1...5....0....5....0....5....0....5....0\n");
		fflush (somewhere);
		grade = 5.0;
		while (grade <= 10.0) {
			plhthos = number_of_lessons_with_grade_and_section ( Tree, grade, 0);
			if ((grade - floor (grade) == 0.5) && (plhthos == 0)) {
				grade += 0.5;
				continue;		// Do not show special cases if not necessary!
			}
			fprintf (somewhere, "%4.1f:   ", grade);;
			for (i = 1; i <= plhthos; i++)
				fprintf (somewhere, "*");
			fprintf (somewhere, "\n");
			fflush (somewhere);
			grade += 0.5;
		}
	}
	return 1;		// Indicate successful finish.
}

int number_of_lessons_with_grade_certain_barythta_and_section (TREEPTR Tree, double bathmos, 
															   char *baryt, const int tomeas)
{
	int i = 0;
	TREE_NODEPTR current = TREE__Minimum (Tree, Tree->ROOT);

	while (current != Tree->NIL) {
		if ((current->data->grade == bathmos) && (strcmp(current->data->barythta, baryt) == 0) &&
			(current->data->section == tomeas))
			i++;
		current = TREE__Successor (Tree, current);
	}

	return i;		// Return number of lessons.
}

int number_of_lessons_with_grade_and_certain_barythta (TREEPTR Tree, double bathmos, char *baryt)
{
	int i = 0;
	TREE_NODEPTR current = TREE__Minimum (Tree, Tree->ROOT);

	while (current != Tree->NIL) {
		if ((current->data->grade == bathmos) && (strcmp(current->data->barythta, baryt) == 0))
			i++;
		current = TREE__Successor (Tree, current);
	}

	return i;		// Return number of lessons.
}

int number_of_lessons_with_grade_and_section (TREEPTR Tree, double bathmos, const int tomeas)
{
	int i = 0;
	TREE_NODEPTR current = TREE__Minimum (Tree, Tree->ROOT);

	while (current != Tree->NIL) {
		if ((current->data->grade == bathmos) && (current->data->section == tomeas))
			i++;
		current = TREE__Successor (Tree, current);
	}

	return i;		// Return number of lessons.
}

int number_of_lessons_with_grade (TREEPTR Tree, double bathmos)
{
	int i = 0;
	TREE_NODEPTR current = TREE__Minimum (Tree, Tree->ROOT);

	while (current != Tree->NIL) {
		if (current->data->grade == bathmos)
			i++;
		current = TREE__Successor (Tree, current);
	}

	return i;		// Return number of lessons.
}

/*********************************************************/

int create_text_with_average_fluctuation (TREEPTR Tree, const int tomeas, OWNERPTR owner, char *FileName)
{
	char date_passed [DATE_PASSED_BUFFER];
	char *tokenPtr;
	char temp [DATE_PASSED_BUFFER];
	char barythta [BARYTHTA_BUFFER];
	KEY_TYPE lesson_keys [ARRAY_SIZE];
	LISTPTR february, june, september;
	KEY_TYPE key;
	TREE_NODEPTR current;
	int i, j, k, l, feb, jun, sep;
	double feb_avg, jun_avg, sep_avg;
	int years [ARRAY_SIZE], hmeromhnia [ARRAY_SIZE];
	int xronos;
	FILE *file;
	FLUCTUATION_AVERAGEPTR average = (FLUCTUATION_AVERAGEPTR) malloc (sizeof (FLUCTUATION_AVERAGE));
	average->ypoxrewtika = LIST__Initialize ();
	average->basika = LIST__Initialize ();
	average->epiloghs = LIST__Initialize ();

	file = fopen (FileName, "w");

	if (file == NULL)
		return 0;		// Indicate error occured.

	average->number_of_lessons = 0;
	average->arithmiths = 0.0;
	average->paronomasths = 0.0;
	average->average_grade = 0.0;

	for (i = 1; i <= ARRAY_SIZE; i++) {
		years [i - 1] = 0;
	}

	current = TREE__Minimum (Tree, Tree->ROOT);
	while (current != Tree->NIL) {
		strcpy (date_passed, current->data->date_passed);
		tokenPtr = strtok (date_passed, "/");
		for (i = 1; i <= 2; i++) {
			tokenPtr = strtok (NULL, "/");
		}
		strcpy (temp, tokenPtr);
		xronos = atoi (temp);
		insert_year (xronos, years);
		/* Examine next Tree node. */
		current = TREE__Successor (Tree, current);
	}

	i = 0;
	fprintf (file, "***********************************************\n");
	fprintf (file, "* UNIVERSITY OF ATHENS                        *\n");
	fprintf (file, "* DEPT. OF INFORMATICS AND TELECOMMUNICATIONS *\n");
	fprintf (file, "***********************************************\n");
	fprintf (file, "\n\n");
	fprintf (file, "STUDENT CHARACTERISTICS\n");
	fprintf (file, "-----------------------\n");
	fprintf (file, "First Name: %s\n", owner->firstname);
	fprintf (file, "Last Name : %s\n", owner->lastname);
	fprintf (file, "Id. Number: %s\n", owner->id);
	fprintf (file, "Entry Date: %s\n", owner->entry_date);
	fprintf (file, "Birthday  : %s\n", owner->birthday);
	fprintf (file, "\n\n");
	if (tomeas == 0) {
		fprintf (file, "                            UNI-SECTION AVERAGE GRADE FLUCTUATION\n");
		fprintf (file, "                            -------------------------------------\n");
	}
	else {
		switch (tomeas) {
			case 1:
				fprintf (file, "                       AVERAGE GRADE FLUCTUATION FOLLOWING 1st SECTION\n");
				fprintf (file, "                       -----------------------------------------------\n");
				break;
			case 2:
				fprintf (file, "                       AVERAGE GRADE FLUCTUATION FOLLOWING 2nd SECTION\n");
				fprintf (file, "                       -----------------------------------------------\n");
				break;
			case 3:
				fprintf (file, "                       AVERAGE GRADE FLUCTUATION FOLLOWING 3rd SECTION\n");
				fprintf (file, "                       -----------------------------------------------\n");
				break;
		}
	}
	fprintf (file, "EXAMINATION PERIOD                                                     \n");
	fprintf (file, "------------------   5....|....6....|....7....|....8....|....9....|....10\n");
	while (years [i] != 0) {
		for (k = 1; k <= ARRAY_SIZE; k++) {
			hmeromhnia [k - 1] = 1300;		//Impossible to be correct.
		}
		find_lesson_keys (years [i], Tree, lesson_keys);	//Find lessons passed this year.

		j = 0;
		while (lesson_keys [j] != 0) {
			key  = lesson_keys [j];
			current = TREE__Search (Tree, key);
			strcpy (date_passed, current->data->date_passed);
			tokenPtr = strtok (date_passed, "/");
			for (k = 1; k <= 2; k++) {
				strcpy (temp, tokenPtr);
				if (k == 1)
					hmeromhnia [j] = atoi (temp);
				else
					hmeromhnia [j] += 100 * atoi (temp);
				tokenPtr = strtok (NULL, "/");
			}
			j++;
		}
		QuickSort_int (hmeromhnia, 0, ARRAY_SIZE - 1, lesson_keys);
		/* Now lesson_keys are sorted in increasing days of year. */

		february = LIST__Initialize ();
		june = LIST__Initialize ();
		september = LIST__Initialize ();
		j = 0;
		feb = 0;
		jun = 0;
		sep = 0;
		while (lesson_keys [j] != 0) {
			key = lesson_keys [j];
			current = TREE__Search (Tree, key);
			strcpy (barythta, current->data->barythta);
			if ((hmeromhnia [j] > 100) && (hmeromhnia [j] < 400)) {
				LIST__Insert_node_at_front (february, lesson_keys [j], current);
				feb++;
			}
			if ((hmeromhnia [j] > 500) && (hmeromhnia [j] < 800)) {
				LIST__Insert_node_at_front (june, lesson_keys [j], current);
				jun++;
			}
			if ((hmeromhnia [j] > 800) && (hmeromhnia [j] < 1100)) {
				LIST__Insert_node_at_front (september, lesson_keys [j], current);
				sep++;
			}
			j++;
		}

		manage_average_fluctuation (Tree, february, average, tomeas);
		feb_avg = average->average_grade;

		if (feb < 10)
			fprintf (file, "(%d)  February %-4d  |", feb, years [i]);
		else
			fprintf (file, "(%2d) February %-4d  |", feb, years [i]);
		l = stroggylopoihsh (feb_avg * 10);
		l -= 49;
		for (k = 1; k <= l; k++) {
			fprintf (file, "*");
		}
		fprintf (file, "\n");

		/* H if yparxei gia to endexomeno h eksetastikh Febrouariou toy trexontos */
		/* etous na einai kai h teleytaia eksetastikh gia ton sygkekrimeno foithth */
		if ( ((jun > 0) || (sep > 0)) || (years [i + 1] != 0)) {
			manage_average_fluctuation (Tree, june, average, tomeas);
			jun_avg = average->average_grade;
			if (jun < 10)
				fprintf (file, "(%d)      June %-4d  |", jun, years [i]);
			else
				fprintf (file, "(%2d)     June %-4d  |", jun, years [i]);
			l = stroggylopoihsh (jun_avg * 10);
			l -= 49;
			for (k = 1; k <= l; k++) {
				fprintf (file, "*");
			}
			fprintf (file, "\n");
		}

		if ( (sep > 0) || (years [i + 1] != 0)) {
			manage_average_fluctuation (Tree, september, average, tomeas);
			sep_avg = average->average_grade;
			if (sep < 10)
				fprintf (file, "(%d) September %-4d  |", sep, years [i]);
			else
				fprintf (file, "(%2d)September %-4d  |", sep, years [i]);
			l = stroggylopoihsh (sep_avg * 10);
			l -= 49;
			for (k = 1; k <= l; k++) {
				fprintf (file, "*");
			}
			fprintf (file, "\n");
		}

		/* Dealloccate useless memory */
		LIST__Destroy (february);
		LIST__Destroy (june);
		LIST__Destroy (september);

		/* Go to next year */
		i++;
	}

	/* Deallocate memory */
	LIST__Destroy (average->ypoxrewtika);
	LIST__Destroy (average->basika);
	LIST__Destroy (average->epiloghs);
	free (average);

	fclose (file);

	return 1;	// Indicate succesful finish.
}

int create_text_with_average_stats (TREEPTR Tree, OWNERPTR owner, char *FileName)
{
	char date_passed [DATE_PASSED_BUFFER];
	char *tokenPtr;
	char temp [DATE_PASSED_BUFFER];
	char barythta [BARYTHTA_BUFFER];
	KEY_TYPE lesson_keys [ARRAY_SIZE];
	KEY_TYPE january [ARRAY_SIZE], june [ARRAY_SIZE], september [ARRAY_SIZE];
	KEY_TYPE key;
	TREE_NODEPTR current;
	int i, j, k, l, jan, jun, sep;
	double jan_avg, jun_avg, sep_avg;
	int years [ARRAY_SIZE], hmeromhnia [ARRAY_SIZE];
	int xronos;
	FILE *file;

	file = fopen (DEFAULT_AVERAGE_STATS_FILENAME, "w");

	if (file == NULL)
		return 0;		// Indicate error occured.

	for (i = 1; i <= ARRAY_SIZE; i++) {
		years [i - 1] = 0;
	}

	current = TREE__Minimum (Tree, Tree->ROOT);
	while (current != Tree->NIL) {
		strcpy (date_passed, current->data->date_passed);
		tokenPtr = strtok (date_passed, "/");
		for (i = 1; i <= 2; i++) {
			tokenPtr = strtok (NULL, "/");
		}
		strcpy (temp, tokenPtr);
		xronos = atoi (temp);
		insert_year (xronos, years);
		/* Examine next Tree node. */
		current = TREE__Successor (Tree, current);
	}

	i = 0;
	fprintf (file, "***********************************************\n");
	fprintf (file, "* UNIVERSITY OF ATHENS                        *\n");
	fprintf (file, "* DEPT. OF INFORMATICS AND TELECOMMUNICATIONS *\n");
	fprintf (file, "***********************************************\n");
	fprintf (file, "\n\n");
	fprintf (file, "STUDENT CHARACTERISTICS\n");
	fprintf (file, "-----------------------\n");
	fprintf (file, "First Name: %s\n", owner->firstname);
	fprintf (file, "Last Name : %s\n", owner->lastname);
	fprintf (file, "Id. Number: %s\n", owner->id);
	fprintf (file, "Entry Date: %s\n", owner->entry_date);
	fprintf (file, "Birthday  : %s\n", owner->birthday);
	fprintf (file, "\n\n");
	fprintf (file, "                               AVERAGE GRADE OF LESSONS PASSED\n");
	fprintf (file, "                               -------------------------------\n");
	fprintf (file, "EXAMINATION PERIOD                                                     \n");
	fprintf (file, "------------------   5....|....6....|....7....|....8....|....9....|....10\n");
	while (years [i] != 0) {
		for (k = 1; k <= ARRAY_SIZE; k++) {
			hmeromhnia [k - 1] = 1300;
		}
		find_lesson_keys (years [i], Tree, lesson_keys);	//Find lessons passed this year.

		j = 0;
		while (lesson_keys [j] != 0) {
			key  = lesson_keys [j];
			current = TREE__Search (Tree, key);
			strcpy (date_passed, current->data->date_passed);
			tokenPtr = strtok (date_passed, "/");
			for (k = 1; k <= 2; k++) {
				strcpy (temp, tokenPtr);
				if (k == 1)
					hmeromhnia [j] = atoi (temp);
				else
					hmeromhnia [j] += 100 * atoi (temp);
				tokenPtr = strtok (NULL, "/");
			}
			j++;
		}
		QuickSort_int (hmeromhnia, 0, ARRAY_SIZE - 1, lesson_keys);
		/* Now lesson_keys are sorted in increasing days of year. */

		j = 0;
		for (k = 1; k <= ARRAY_SIZE; k++) {
			january [k - 1] = 0;
			june [k - 1] = 0;
			september [k - 1] = 0;
		}
		jan = 0;
		jun = 0;
		sep = 0;
		while (lesson_keys [j] != 0) {
			key = lesson_keys [j];
			current = TREE__Search (Tree, key);
			strcpy (barythta, current->data->barythta);
			if ((hmeromhnia [j] > 100) && (hmeromhnia [j] < 400)) {
				january [jan] = lesson_keys [j];
				jan++;
			}
			if ((hmeromhnia [j] > 500) && (hmeromhnia [j] < 800)) {
				june [jun] = lesson_keys [j];
				jun++;
			}
			if ((hmeromhnia [j] > 800) && (hmeromhnia [j] < 1100)) {
				september [sep] = lesson_keys [j];
				sep++;
			}
			j++;
		}

		jan_avg = calculate_period_average (Tree, january);
		jun_avg = calculate_period_average (Tree, june);
		sep_avg = calculate_period_average (Tree, september);

		if (jan < 10)
			fprintf (file, "(%d)  February %-4d  |", jan, years [i]);
		else
			fprintf (file, "(%2d) February %-4d  |", jan, years [i]);
		l = stroggylopoihsh (jan_avg * 10);
		l -= 49;
		for (k = 1; k <= l; k++) {
			fprintf (file, "*");
		}
		fprintf (file, "\n");

		/* H if yparxei gia to endexomeno h eksetastikh Febrouariou toy trexontos */
		/* etous na einai kai h teleytaia eksetastikh gia ton sygkekrimeno foithth */
		if ( ((jun > 0) || (sep > 0)) || (years [i + 1] != 0)) {
			if (jun < 10)
				fprintf (file, "(%d)      June %-4d  |", jun, years [i]);
			else
				fprintf (file, "(%2d)     June %-4d  |", jun, years [i]);
			l = stroggylopoihsh (jun_avg * 10);
			l -= 49;
			for (k = 1; k <= l; k++) {
				fprintf (file, "*");
			}
			fprintf (file, "\n");
		}

		if ( (sep > 0) || (years [i + 1] != 0)) {
			if (sep < 10)
				fprintf (file, "(%d) September %-4d  |", sep, years [i]);
			else
				fprintf (file, "(%2d)September %-4d  |", sep, years [i]);
			l = stroggylopoihsh (sep_avg * 10);
			l -= 49;
			for (k = 1; k <= l; k++) {
				fprintf (file, "*");
			}
			fprintf (file, "\n");
		}

		/* Go to next year */
		i++;
	}

	fclose (file);	// Close file.

	return 1;	// Indicate succesful finish.
}

int create_text_with_lesson_stats (TREEPTR Tree, OWNERPTR owner, char *FileName)
{
	char date_passed [DATE_PASSED_BUFFER];
	char *tokenPtr;
	char temp [DATE_PASSED_BUFFER];
	char barythta [BARYTHTA_BUFFER];
	KEY_TYPE lesson_keys [ARRAY_SIZE];
	KEY_TYPE january [ARRAY_SIZE], june [ARRAY_SIZE], september [ARRAY_SIZE];
	KEY_TYPE key;
	TREE_NODEPTR current;
	int i, j, k, jan, jun, sep;
	int jan_kormou, jan_basika, jan_epiloghs, jan_ptyxiakh, 
		jun_kormou, jun_basika, jun_epiloghs, jun_ptyxiakh, 
		sep_kormou, sep_basika, sep_epiloghs, sep_ptyxiakh;
	int synolo;
	int years [ARRAY_SIZE], hmeromhnia [ARRAY_SIZE];
	int xronos;
	FILE *somewhere;

	somewhere = fopen (FileName, "w");

	if (somewhere == NULL)
		return 0;	// Indicate error occured.

	for (i = 1; i <= ARRAY_SIZE; i++) {
		years [i - 1] = 0;
	}

	current = TREE__Minimum (Tree, Tree->ROOT);
	while (current != Tree->NIL) {
		strcpy (date_passed, current->data->date_passed);
		tokenPtr = strtok (date_passed, "/");
		for (i = 1; i <= 2; i++) {
			tokenPtr = strtok (NULL, "/");
		}
		strcpy (temp, tokenPtr);
		xronos = atoi (temp);
		insert_year (xronos, years);
		/* Examine next Tree node. */
		current = TREE__Successor (Tree, current);
	}

	i = 0;
	fprintf (somewhere, "***********************************************\n");
	fprintf (somewhere, "* UNIVERSITY OF ATHENS                        *\n");
	fprintf (somewhere, "* DEPT. OF INFORMATICS AND TELECOMMUNICATIONS *\n");
	fprintf (somewhere, "***********************************************\n");
	fprintf (somewhere, "\n\n");
	fprintf (somewhere, "STUDENT CHARACTERISTICS\n");
	fprintf (somewhere, "-----------------------\n");
	fprintf (somewhere, "First Name: %s\n", owner->firstname);
	fprintf (somewhere, "Last Name : %s\n", owner->lastname);
	fprintf (somewhere, "Id. Number: %s\n", owner->id);
	fprintf (somewhere, "Entry Date: %s\n", owner->entry_date);
	fprintf (somewhere, "Birthday  : %s\n", owner->birthday);
	fprintf (somewhere, "\n\n");
	fprintf (somewhere, "                     NUMBER OF LESSONS PASSED\n");
	fprintf (somewhere, "                     ------------------------\n");
	fprintf (somewhere, "EXAMINATION PERIOD              1    1    2  \tKIND OF LESSONS PASSED\n");
	fprintf (somewhere, "------------------    0    5    0    5    0  \t----------------------\n");
	while (years [i] != 0) {
		for (k = 1; k <= ARRAY_SIZE; k++) {
			hmeromhnia [k - 1] = 1300;
		}
		find_lesson_keys (years [i], Tree, lesson_keys);
		j = 0;
		while (lesson_keys [j] != 0) {
			key  = lesson_keys [j];
			current = TREE__Search (Tree, key);
			strcpy (date_passed, current->data->date_passed);
			tokenPtr = strtok (date_passed, "/");
			for (k = 1; k <= 2; k++) {
				strcpy (temp, tokenPtr);
				if (k == 1)
					hmeromhnia [j] = atoi (temp);
				else
					hmeromhnia [j] += 100 * atoi (temp);
				tokenPtr = strtok (NULL, "/");
			}
			j++;
		}
		QuickSort_int (hmeromhnia, 0, ARRAY_SIZE - 1, lesson_keys);
		/* Now lesson_keys are sorted in increasing days of year. */

		j = 0;
		for (k = 1; k <= ARRAY_SIZE; k++) {
			january [k - 1] = 0;
			june [k - 1] = 0;
			september [k - 1] = 0;
		}
		jan = 0;
		jun = 0;
		sep = 0;
		jan_kormou = 0;
		jan_basika = 0;
		jan_epiloghs = 0;
		jan_ptyxiakh = 0;
		jun_kormou = 0;
		jun_basika = 0;
		jun_epiloghs = 0;
		jun_ptyxiakh = 0;
		sep_kormou = 0;
		sep_basika = 0;
		sep_epiloghs = 0;
		sep_ptyxiakh = 0;
		while (lesson_keys [j] != 0) {
			key = lesson_keys [j];
			current = TREE__Search (Tree, key);
			strcpy (barythta, current->data->barythta);
			if ((hmeromhnia [j] > 100) && (hmeromhnia [j] < 400)) {
				january [jan] = lesson_keys [j];
				if (strcmp (barythta, "YP") == 0)
					jan_kormou++;
				if (strcmp (barythta, "BK") == 0)
					jan_basika++;
				if (strcmp (barythta, "EP") == 0)
					jan_epiloghs++;
				if (strcmp (barythta, "PT") == 0)
					jan_ptyxiakh++;
				jan++;
			}
			if ((hmeromhnia [j] > 500) && (hmeromhnia [j] < 800)) {
				june [jun] = lesson_keys [j];
				if (strcmp (barythta, "YP") == 0)
					jun_kormou++;
				if (strcmp (barythta, "BK") == 0)
					jun_basika++;
				if (strcmp (barythta, "EP") == 0)
					jun_epiloghs++;
				if (strcmp (barythta, "PT") == 0)
					jun_ptyxiakh++;
				jun++;
			}
			if ((hmeromhnia [j] > 800) && (hmeromhnia [j] < 1100)) {
				september [sep] = lesson_keys [j];
				if (strcmp (barythta, "YP") == 0)
					sep_kormou++;
				if (strcmp (barythta, "BK") == 0)
					sep_basika++;
				if (strcmp (barythta, "EP") == 0)
					sep_epiloghs++;
				if (strcmp (barythta, "PT") == 0)
					sep_ptyxiakh++;
				sep++;
			}
			j++;
		}

		fprintf (somewhere, "     February %-4d    |", years [i]);
		for (k = 1; k <= 22; k++) {
			if (k <= jan)
				fprintf (somewhere, "*");
			else
				fprintf (somewhere, " ");
		}
		fprintf (somewhere, "\t");
		synolo = jan_kormou + jan_basika + jan_epiloghs + jan_ptyxiakh;
		if (synolo > 0) {
			if ((jan_kormou < synolo) && (jan_kormou > 0))
				fprintf (somewhere, "YP=%d, ", jan_kormou);
			else {
				if (jan_kormou)
					fprintf (somewhere, "YP=%d", jan_kormou);
			}
			if ((jan_kormou + jan_basika < synolo) && (jan_basika > 0))
				fprintf (somewhere, "BK=%d, ", jan_basika);
			else {
				if (jan_basika)
					fprintf (somewhere, "BK=%d", jan_basika);
			}
			if ((jan_kormou + jan_basika + jan_epiloghs < synolo) && (jan_epiloghs > 0))
				fprintf (somewhere, "EP=%d, ", jan_epiloghs);
			else {
				if (jan_epiloghs)
					fprintf (somewhere, "EP=%d", jan_epiloghs);
			}
			if (jan_ptyxiakh)
				fprintf (somewhere, "PTYXIAKH");
		}
		fprintf (somewhere, "\n");

		/* H if yparxei gia to endexomeno h eksetastikh Febrouariou toy trexontos */
		/* etous na einai kai h teleytaia eksetastikh gia ton sygkekrimeno foithth */
		if ( ((jun > 0) || (sep > 0)) || (years [i + 1] != 0)) {
			fprintf (somewhere, "         June %-4d    |", years [i]);
			for (k = 1; k <= 22; k++) {
				if (k <= jun)
					fprintf (somewhere, "*");
				else
					fprintf (somewhere, " ");
			}
			fprintf (somewhere, "\t");
			synolo = jun_kormou + jun_basika + jun_epiloghs + jun_ptyxiakh;
			if (synolo > 0) {
				if ((jun_kormou < synolo) && (jun_kormou > 0))
					fprintf (somewhere, "YP=%d, ", jun_kormou);
				else {
					if (jun_kormou)
						fprintf (somewhere, "YP=%d", jun_kormou);
				}
				if ((jun_kormou + jun_basika < synolo) && (jun_basika > 0))
					fprintf (somewhere, "BK=%d, ", jun_basika);
				else {
					if (jun_basika)
						fprintf (somewhere, "BK=%d", jun_basika);
				}
				if ((jun_kormou + jun_basika + jun_epiloghs < synolo) && (jun_epiloghs > 0))
					fprintf (somewhere, "EP=%d, ", jun_epiloghs);
				else {
					if (jun_epiloghs)
						fprintf (somewhere, "EP=%d", jun_epiloghs);
				}
				if (jun_ptyxiakh)
					fprintf (somewhere, "PTYXIAKH");
			}
			fprintf (somewhere, "\n");
		}

		if ( (sep > 0) || (years [i + 1] != 0)) {
			fprintf (somewhere, "    September %-4d    |", years [i]);
			for (k = 1; k <= 22; k++) {
				if (k <= sep)
					fprintf (somewhere, "*");
				else
					fprintf (somewhere, " ");
			}
			fprintf (somewhere, "\t");
			synolo = sep_kormou + sep_basika + sep_epiloghs + sep_ptyxiakh;
			if (synolo > 0) {
				if ((sep_kormou < synolo) && (sep_kormou > 0))
					fprintf (somewhere, "YP=%d, ", sep_kormou);
				else {
					if (sep_kormou)
						fprintf (somewhere, "YP=%d", sep_kormou);
				}
				if ((sep_kormou + sep_basika < synolo) && (sep_basika > 0))
					fprintf (somewhere, "BK=%d, ", sep_basika);
				else {
					if (sep_basika)
						fprintf (somewhere, "BK=%d", sep_basika);
				}
				if ((sep_kormou + sep_basika + sep_epiloghs < synolo) && (sep_epiloghs > 0))
					fprintf (somewhere, "EP=%d, ", sep_epiloghs);
				else {
					if (sep_epiloghs)
						fprintf (somewhere, "EP=%d", sep_epiloghs);
				}
				if (sep_ptyxiakh)
					fprintf (somewhere, "PTYXIAKH");
			}
			fprintf (somewhere, "\n");
		}

		/* Go to next year */
		i++;
	}

	fclose (somewhere);

	return 1;	// Indicate succesful finish.
}

int create_text_with_tree_keys (TREEPTR Tree, OWNERPTR owner, char *FileName)
{
	FILE *somewhere;
	int result;

	somewhere = fopen (FileName, "w");

	if (somewhere == NULL)
		return 0;	// Indicate error occured.

	fprintf (somewhere, "***********************************************\n");
	fprintf (somewhere, "* UNIVERSITY OF ATHENS                        *\n");
	fprintf (somewhere, "* DEPT. OF INFORMATICS AND TELECOMMUNICATIONS *\n");
	fprintf (somewhere, "***********************************************\n");
	fprintf (somewhere, "\n\n");
	fprintf (somewhere, "STUDENT CHARACTERISTICS\n");
	fprintf (somewhere, "-----------------------\n");
	fprintf (somewhere, "First Name: %s\n", owner->firstname);
	fprintf (somewhere, "Last Name : %s\n", owner->lastname);
	fprintf (somewhere, "Id. Number: %s\n", owner->id);
	fprintf (somewhere, "Entry Date: %s\n", owner->entry_date);
	fprintf (somewhere, "Birthday  : %s\n", owner->birthday);
	fprintf (somewhere, "\n\n");
	fprintf (somewhere, "Tree's Nodes and Leaves with their corresponding Keys:\n\n");
	result = create_subtree (somewhere, Tree, Tree->ROOT, 0, KEY_DISPLAY);

	fclose (somewhere);

	return result;	// Indicate succesful finish.

}

int create_text_with_tree_colors (TREEPTR Tree, OWNERPTR owner, char *FileName)
{
	FILE *somewhere;
	int result;

	somewhere = fopen (FileName, "w");

	if (somewhere == NULL)
		return 0;	// Indicate error occured.

	fprintf (somewhere, "***********************************************\n");
	fprintf (somewhere, "* UNIVERSITY OF ATHENS                        *\n");
	fprintf (somewhere, "* DEPT. OF INFORMATICS AND TELECOMMUNICATIONS *\n");
	fprintf (somewhere, "***********************************************\n");
	fprintf (somewhere, "\n\n");
	fprintf (somewhere, "STUDENT CHARACTERISTICS\n");
	fprintf (somewhere, "-----------------------\n");
	fprintf (somewhere, "First Name: %s\n", owner->firstname);
	fprintf (somewhere, "Last Name : %s\n", owner->lastname);
	fprintf (somewhere, "Id. Number: %s\n", owner->id);
	fprintf (somewhere, "Entry Date: %s\n", owner->entry_date);
	fprintf (somewhere, "Birthday  : %s\n", owner->birthday);
	fprintf (somewhere, "\n\n");
	fprintf (somewhere, "Tree's Nodes and Leaves with their corresponding Colors:\n\n");
	result = create_subtree (somewhere, Tree, Tree->ROOT, 0, COLOR_DISPLAY);

	fclose (somewhere);

	return result;	// Indicate succesful finish.

}

int create_subtree (FILE *somewhere, TREEPTR Tree, TREE_NODEPTR Node, int indent_so_far, const int display_selection)
{
	int i;
	int new_indent;
	
	if (display_selection == KEY_DISPLAY)
		new_indent = KEY_DISPLAY_INDENT + indent_so_far;
	else
		new_indent = COLOR_DISPLAY_INDENT + indent_so_far;

	if (Node->right_child != Tree->NIL) {
		create_subtree (somewhere, Tree, Node->right_child, new_indent, display_selection);
	}


	for (i = 1; i<= indent_so_far; i++)
		fprintf (somewhere, " ");
	if (display_selection == KEY_DISPLAY)
		fprintf (somewhere, "%6d\n", Node->key);
	else
		fprintf (somewhere, "%s\n", (Node->color == RED ? "RED" : "BLACK"));


	if (Node->left_child != Tree->NIL) {
		create_subtree (somewhere, Tree, Node->left_child, new_indent, display_selection);
		fprintf (somewhere, "\n");
	}


	return 1;			// Indicate successful display.
}

int create_text_with_tree (TREEPTR Tree, OWNERPTR owner, char *FileName)
{
	FILE *somewhere;
	int result;

	somewhere = fopen (FileName, "w");

	if (somewhere == NULL)
		return 0;	// Indicate error occured.

	fprintf (somewhere, "***********************************************\n");
	fprintf (somewhere, "* UNIVERSITY OF ATHENS                        *\n");
	fprintf (somewhere, "* DEPT. OF INFORMATICS AND TELECOMMUNICATIONS *\n");
	fprintf (somewhere, "***********************************************\n");
	fprintf (somewhere, "\n\n");
	fprintf (somewhere, "STUDENT CHARACTERISTICS\n");
	fprintf (somewhere, "-----------------------\n");
	fprintf (somewhere, "First Name: %s\n", owner->firstname);
	fprintf (somewhere, "Last Name : %s\n", owner->lastname);
	fprintf (somewhere, "Id. Number: %s\n", owner->id);
	fprintf (somewhere, "Entry Date: %s\n", owner->entry_date);
	fprintf (somewhere, "Birthday  : %s\n", owner->birthday);
	fprintf (somewhere, "\n\n");
	fprintf (somewhere, "Tree's Nodes and Leaves with their corresponding Keys and Colors:\n\n");
	result = create_subtree_keys_colors (somewhere, Tree, Tree->ROOT, 0);

	fclose (somewhere);

	return result;	// Indicate succesful finish.

}

int create_subtree_keys_colors (FILE *somewhere, TREEPTR Tree, TREE_NODEPTR Node, int indent_so_far)
{
	int i;
	int new_indent;
	
	new_indent = (KEY_DISPLAY_INDENT + COLOR_DISPLAY_INDENT - 1) + indent_so_far;

	if (Node->right_child != Tree->NIL) {
		create_subtree_keys_colors (somewhere, Tree, Node->right_child, new_indent);
	}


	for (i = 1; i<= indent_so_far; i++)
		fprintf (somewhere, " ");
	fprintf (somewhere, "%6d-%s\n", Node->key, (Node->color == RED ? "RED" : "BLACK"));


	if (Node->left_child != Tree->NIL) {
		create_subtree_keys_colors (somewhere, Tree, Node->left_child, new_indent);
		fprintf (somewhere, "\n");
	}


	return 1;			// Indicate successful display.
}

/*********************************************************/

int find_lesson_keys (int xronia, TREEPTR Tree, KEY_TYPE keys [ARRAY_SIZE])
{
	int i, xronos;
	char date_passed [DATE_PASSED_BUFFER], temp [DATE_PASSED_BUFFER];
	char *tokenPtr;
	TREE_NODEPTR current;

	for (i = 1; i <= ARRAY_SIZE; i++) {
		keys [i - 1] = 0;
	}
	current = TREE__Minimum (Tree, Tree->ROOT);
	while (current != Tree->NIL) {
		strcpy (date_passed, current->data->date_passed);
		tokenPtr = strtok (date_passed, "/");
		for (i = 1; i <= 2; i++) {
			tokenPtr = strtok (NULL, "/");
		}
		strcpy (temp, tokenPtr);
		xronos = atoi (temp);
		if (xronos == xronia)
			insert_key (current->key, keys);
		current = TREE__Successor (Tree, current);
	}

	return 1;	// Indicate successful finish.
}

void insert_key (KEY_TYPE key, KEY_TYPE array_of_keys [ARRAY_SIZE])
{
	int i = 0;

	while (array_of_keys [i] != 0) {
		i++;
	}
	array_of_keys [i] = key;
}

/*********************************************************/

void insert_year (int year, int array_of_years [ARRAY_SIZE])
{
	int i = 0;

	if (!belongs (year, array_of_years)) {
		while (array_of_years [i] != 0) {
			i++;
		}
		array_of_years [i] = year;

		/* Theloume taksinomhsh mexri th 8esh "i" */
		QuickSort_single_int (array_of_years, 0, i);
	}
}

int belongs (int something, int array_of_something [ARRAY_SIZE])
{
	int i = 0, found = 0;

	while (array_of_something [i] != 0) {
		if (array_of_something [i] == something) {
			found = 1;
			break;
		}
		i++;
	}

	if (found)
		return 1;	// Indicate that somthing belongs to the array.

	return 0;		// Indicate that somthing does't belong to the array.
}

/*********************************************************/

int update_file (char *FileName, TREEPTR Tree, OWNERPTR owner)
{
	FILE *outputFile;

	outputFile = fopen (FileName, "w");
	if (outputFile == NULL) {
		printf ("\nError creating file!\n\a");
		return 0;	/* Indicate UNsuccessful update. */
	}
	else {
		write_owner_to_file (outputFile, owner);
		write_POSTORDER_tree_node_data_to_file (outputFile, Tree, Tree->ROOT);
		fprintf (outputFile, "%c", EOF);
	}

	fclose (outputFile);

	return 1;	/* Indicate successful update. */
}

int secure_update (char *FileName, TREEPTR Tree, OWNERPTR owner)
{
	FILE *outputFile;

	CLEARSCREEN;
	show_logo ();

	printf ("Creating temporary backup file. Please wait...");
	fflush (stdout);

	CREATE_BACKUP_FILE;

	printf ("\nBackup file was created.\n");
	printf ("Creating new and up-to-date datafile. Please wait...");
	fflush (stdout);

	outputFile = fopen (FileName, "w");
	if (outputFile == NULL) {
		return 0;	/* Indicate UNsuccessful update. */
	}
	else {
		printf ("\nWriting owner properties to file...");
		fflush (stdout);
		write_owner_to_file (outputFile, owner);
		printf ("\nWriting RED-BLACK Tree elements POSTORDER to file. Please wait...");
		fflush (stdout);
		write_POSTORDER_tree_node_data_to_file (outputFile, Tree, Tree->ROOT);
		fprintf (outputFile, "%c", EOF);
		printf ("\nCompleted writing to file.\n");
	}

	fclose (outputFile);
	printf ("File closed.\n");

	printf ("Removing temporary backup file. Please wait...");
	fflush (stdout);

	DELETE_BACKUP_FILE;

	printf ("Backup file was removed.\n\n");
	printf ("SECURE UPDATE COMPLETED SUCCESSFULLY.");
	fflush (stdout);

	return 1;
}

int write_INORDER_tree_node_data_to_file (FILE *outputFile, TREEPTR Tree, TREE_NODEPTR x)
{
	/* INORDER WRITING */

	if (x != Tree->NIL) {
		write_INORDER_tree_node_data_to_file (outputFile, Tree, x->left_child);
		fwrite (x->data, sizeof (DATA_TYPE), 1, outputFile);
		write_INORDER_tree_node_data_to_file (outputFile, Tree, x->right_child);
	}

	return 1;	/* Indicate successful writing to file. */
}

int write_POSTORDER_tree_node_data_to_file (FILE *outputFile, TREEPTR Tree, TREE_NODEPTR x)
{
	/* POSTORDER WRITING */

	if (x != Tree->NIL) {
		write_POSTORDER_tree_node_data_to_file (outputFile, Tree, x->left_child);
		write_POSTORDER_tree_node_data_to_file (outputFile, Tree, x->right_child);
		fwrite (x->data, sizeof (DATA_TYPE), 1, outputFile);
	}

	return 1;	/* Indicate successful writing to file */
}

/*********************************************************/

int write_average (AVERAGE_REPLYPTR average, OWNERPTR owner, FILE *somewhere)
{
	int i;

	fprintf (somewhere, "***********************************************\n");
	fprintf (somewhere, "* UNIVERSITY OF ATHENS                        *\n");
	fprintf (somewhere, "* DEPT. OF INFORMATICS AND TELECOMMUNICATIONS *\n");
	fprintf (somewhere, "***********************************************\n");
	fprintf (somewhere, "\n\n");
	if (somewhere != stdout) {
		fprintf (somewhere, "STUDENT CHARACTERISTICS\n");
		fprintf (somewhere, "-----------------------\n");
		fprintf (somewhere, "First Name: %s\n", owner->firstname);
		fprintf (somewhere, "Last Name : %s\n", owner->lastname);
		fprintf (somewhere, "Id. Number: %s\n", owner->id);
		fprintf (somewhere, "Entry Date: %s\n", owner->entry_date);
		fprintf (somewhere, "Birthday  : %s\n", owner->birthday);
		fprintf (somewhere, "\n\n");
	}
	fprintf (somewhere, "                         Uni-Section  Graduation\n");
	fprintf (somewhere, "                         -----------------------\n");
	fprintf (somewhere, "                        %7.3f\n", average->uniform->arithmiths);
	fprintf (somewhere, "Your Average Grade is: --------- = %.3f\n", average->uniform->average_grade);
	fprintf (somewhere, "                        %7.3f\n", average->uniform->paronomasths);
	fprintf (somewhere, "\n");
	fprintf (somewhere, "Basic-Lessons' Codes    :  ");
	for (i = 1; i<= 5; i++) {
		if (average->uniform->basika [i - 1] != 0)
			fprintf (somewhere, "%ld ", average->uniform->basika [i - 1]);
	}
	fprintf (somewhere, "\n");
	fprintf (somewhere, "Non-Basic-Lessons' Codes:  ");
	for (i = 1; i <= 11; i++) {
		if (average->uniform->epiloghs [i - 1] != 0)
			fprintf (somewhere, "%ld ", average->uniform->epiloghs [i - 1]);
		if (i == 6)
			fprintf (somewhere, "\n                           ");
	}
	fprintf (somewhere, "\n");
	fprintf (somewhere, "Predictions\n");
	fprintf (somewhere, "-----------\n");
	if (average->uniform->get_ptyxio) {
		fprintf (somewhere, "You can graduate my friend (not following a certain section)!\n");
		fprintf (somewhere, "Congratulations!\n");
	}
	else {
		fprintf (somewhere, "Lowest Average Grade : %.3f\n", average->uniform->lowest_grade);
		fprintf (somewhere, "Highest Average Grade: %.3f\n", average->uniform->highest_grade);
	}
	fprintf (somewhere, "\n\n");

	fprintf (somewhere, "                          Following 1st Section\n");
	fprintf (somewhere, "                          ---------------------\n");
	fprintf (somewhere, "                        %7.3f\n", average->prwtos_tomeas->arithmiths);
	fprintf (somewhere, "Your Average Grade is: --------- = %.3f\n", average->prwtos_tomeas->average_grade);
	fprintf (somewhere, "                        %7.3f\n", average->prwtos_tomeas->paronomasths);
	fprintf (somewhere, "\n");
	fprintf (somewhere, "Basic-Lessons' Codes    :  ");
	for (i = 1; i<= 5; i++) {
		if (average->prwtos_tomeas->basika [i - 1] != 0)
			fprintf (somewhere, "%ld ", average->prwtos_tomeas->basika [i - 1]);
	}
	fprintf (somewhere, "\n");
	fprintf (somewhere, "Non-Basic-Lessons' Codes:  ");
	for (i = 1; i <= 11; i++) {
		if (average->prwtos_tomeas->epiloghs [i - 1] != 0)
			fprintf (somewhere, "%ld ", average->prwtos_tomeas->epiloghs [i - 1]);
		if (i == 6)
			fprintf (somewhere, "\n                           ");
	}
	fprintf (somewhere, "\n");
	fprintf (somewhere, "Predictions\n");
	fprintf (somewhere, "-----------\n");
	if (average->prwtos_tomeas->get_ptyxio) {
		fprintf (somewhere, "You can graduate my friend following 1st Section!\n");
		fprintf (somewhere, "Congratulations!\n");
	}
	else {
		fprintf (somewhere, "Lowest Average Grade : %.3f\n", average->prwtos_tomeas->lowest_grade);
		fprintf (somewhere, "Highest Average Grade: %.3f\n", average->prwtos_tomeas->highest_grade);
	}
	fprintf (somewhere, "\n\n");

	fprintf (somewhere, "                          Following 2nd Section\n");
	fprintf (somewhere, "                          ---------------------\n");
	fprintf (somewhere, "                        %7.3f\n", average->deyteros_tomeas->arithmiths);
	fprintf (somewhere, "Your Average Grade is: --------- = %.3f\n", average->deyteros_tomeas->average_grade);
	fprintf (somewhere, "                        %7.3f\n", average->deyteros_tomeas->paronomasths);
	fprintf (somewhere, "\n");
	fprintf (somewhere, "Basic-Lessons' Codes    :  ");
	for (i = 1; i<= 5; i++) {
		if (average->deyteros_tomeas->basika [i - 1] != 0)
			fprintf (somewhere, "%ld ", average->deyteros_tomeas->basika [i - 1]);
	}
	fprintf (somewhere, "\n");
	fprintf (somewhere, "Non-Basic-Lessons' Codes:  ");
	for (i = 1; i <= 11; i++) {
		if (average->deyteros_tomeas->epiloghs [i - 1] != 0)
			fprintf (somewhere, "%ld ", average->deyteros_tomeas->epiloghs [i - 1]);
		if (i == 6)
			fprintf (somewhere, "\n                           ");
	}
	fprintf (somewhere, "\n");
	fprintf (somewhere, "Predictions\n");
	fprintf (somewhere, "-----------\n");
	if (average->deyteros_tomeas->get_ptyxio) {
		fprintf (somewhere, "You can graduate my friend following 2nd Section!\n");
		fprintf (somewhere, "Congratulations!\n");
	}
	else {
		fprintf (somewhere, "Lowest Average Grade : %.3f\n", average->deyteros_tomeas->lowest_grade);
		fprintf (somewhere, "Highest Average Grade: %.3f\n", average->deyteros_tomeas->highest_grade);
	}
	fprintf (somewhere, "\n\n");

	fprintf (somewhere, "                          Following 3rd Section\n");
	fprintf (somewhere, "                          ---------------------\n");
	fprintf (somewhere, "                        %7.3f\n", average->tritos_tomeas->arithmiths);
	fprintf (somewhere, "Your Average Grade is: --------- = %.3f\n", average->tritos_tomeas->average_grade);
	fprintf (somewhere, "                        %7.3f\n", average->tritos_tomeas->paronomasths);
	fprintf (somewhere, "\n");
	fprintf (somewhere, "Basic-Lessons' Codes    :  ");
	for (i = 1; i<= 5; i++) {
		if (average->tritos_tomeas->basika [i - 1] != 0)
			fprintf (somewhere, "%ld ", average->tritos_tomeas->basika [i - 1]);
	}
	fprintf (somewhere, "\n");
	fprintf (somewhere, "Non-Basic-Lessons' Codes:  ");
	for (i = 1; i <= 11; i++) {
		if (average->tritos_tomeas->epiloghs [i - 1] != 0)
			fprintf (somewhere, "%ld ", average->tritos_tomeas->epiloghs [i - 1]);
		if (i == 6)
			fprintf (somewhere, "\n                           ");
	}
	fprintf (somewhere, "\n");
	fprintf (somewhere, "Predictions\n");
	fprintf (somewhere, "-----------\n");
	if (average->tritos_tomeas->get_ptyxio) {
		fprintf (somewhere, "You can graduate my friend following 3rd Section!\n");
		fprintf (somewhere, "Congratulations!\n");
	}
	else {
		fprintf (somewhere, "Lowest Average Grade : %.3f\n", average->tritos_tomeas->lowest_grade);
		fprintf (somewhere, "Highest Average Grade: %.3f\n", average->tritos_tomeas->highest_grade);
	}

	return 1;	// Indicate successful presentation.
}

/*********************************************************/

int create_analytikh_textfile (char *FileName, OWNERPTR owner, TREEPTR Tree)
{
	TREE_NODEPTR current;
	AVERAGE_REPLYPTR average;
	FILE *textfile;
	char code [(6 + BARYTHTA_BUFFER + 3)];

	textfile = fopen (FileName, "w");

	if (textfile == NULL)
		return 0;	// Indicate unsuccessful file creation!

	fprintf (textfile, "***********************************************\n");
	fprintf (textfile, "* UNIVERSITY OF ATHENS                        *\n");
	fprintf (textfile, "* DEPT. OF INFORMATICS AND TELECOMMUNICATIONS *\n");
	fprintf (textfile, "***********************************************\n");
	fprintf (textfile, "\n\n");
	fprintf (textfile, "STUDENT CHARACTERISTICS\n");
	fprintf (textfile, "-----------------------\n");
	fprintf (textfile, "First Name: %s\n", owner->firstname);
	fprintf (textfile, "Last Name : %s\n", owner->lastname);
	fprintf (textfile, "Id. Number: %s\n", owner->id);
	fprintf (textfile, "Entry Date: %s\n", owner->entry_date);
	fprintf (textfile, "Birthday  : %s\n", owner->birthday);
	fprintf (textfile, "\n\n");
	
	fprintf (textfile, "\t\t\t\tANALYTIKH BATHMOLOGIA\n");
	fprintf (textfile, "\t\t\t\t---------------------\n");
	fprintf (textfile, "\n\n");
	fprintf (textfile, "%s\t%-50s\t%s\t%-11s\n", 
						"LESSON CODE", "LESSON NAME", "GRADE", "DATE PASSED");
	fprintf (textfile, "%s\t%-50s\t%s\t%-11s\n", 
						"-----------", "-----------", "-----", "-----------");

	current = TREE__Minimum (Tree, Tree->ROOT);
	while (current != Tree->NIL) {
		sprintf (code, "%-6ld (%s)\0", current->data->code, current->data->barythta);
		fprintf (textfile, "%-11s\t%-50s\t%5.2f\t%-11s\n", 
							code, current->data->lesson, current->data->grade, 
							current->data->date_passed);
		current = TREE__Successor (Tree, current);
	}

	average = calculate_average (Tree);

	fprintf (textfile, "\n\n");
	fprintf (textfile, "AVERAGE GRADE\n");
	fprintf (textfile, "-------------\n");
	fprintf (textfile, " UNI-SECTION:\t%.12f\n", average->uniform->average_grade);
	fprintf (textfile, " 1st-SECTION:\t%.12f\n", average->prwtos_tomeas->average_grade);
	fprintf (textfile, " 2nd-SECTION:\t%.12f\n", average->deyteros_tomeas->average_grade);
	fprintf (textfile, " 3rd-SECTION:\t%.12f\n", average->tritos_tomeas->average_grade);

	fprintf (textfile, "\n");
	fprintf (textfile, "**** For more information on Average Grade consult \"average.txt\" file. ****");

	fclose (textfile);	// Close text file.

	/* Dealloccate unused memory */
	free (average->prwtos_tomeas);
	free (average->deyteros_tomeas);
	free (average->tritos_tomeas);
	free (average->uniform);
	free (average);

	return 1;		// Indicate successful file creation!
}

/*********************************************************/