#include "quicksort.h"

/*
 * Note: This version of QuickSort
 * is modified to sort elements in
 * DECREASING order.
 * If you want to sort elements in
 * INCREASING order modify Partition
 * function in the do { ...} while ()
 * loops and reverse the inequalities.
 */

void QuickSort (double grades [ARRAY_SIZE], int p, int r)
{
	int q;

	if (p < r) {
		q = Partition (grades, p, r);
		QuickSort (grades, p, q);
		QuickSort (grades, (q + 1), r);
	}
}

int Partition (double grades [ARRAY_SIZE], int p, int r)
{
	double x, temp;
	int i, j;

	x = grades [p];
	i = p - 1;
	j = r + 1;
	
	while (1) {
		do {
			j--;
		} while (grades [j] < x);
		do {
			i++;
		} while (grades [i] > x);
		if (i < j) {
			temp = grades [i];
			grades [i] = grades [j];
			grades [j] = temp;
		}
		else {
			break;
		}
	}

	return j;
}

/*********************************************************/

/* This is INCREASING Order */

void QuickSort_single_int (int years [ARRAY_SIZE], int p, int r)
{
	int q;

	if (p < r) {
		q = Partition_single_int (years, p, r);
		QuickSort_single_int (years, p, q);
		QuickSort_single_int (years, (q + 1), r);
	}
}

int Partition_single_int (int years [ARRAY_SIZE], int p, int r)
{
	int x, temp;
	int i, j;

	x = years [p];
	i = p - 1;
	j = r + 1;
	
	while (1) {
		do {
			j--;
		} while (years [j] > x);
		do {
			i++;
		} while (years [i] < x);
		if (i < j) {
			temp = years [i];
			years [i] = years [j];
			years [j] = temp;
		}
		else {
			break;
		}
	}

	return j;
}

/*********************************************************/

void QuickSort_int (int array1 [ARRAY_SIZE], int p, int r, long int array2 [ARRAY_SIZE])
{
	int q;

	if (p < r) {
		q = Partition_int (array1, p, r, array2);
		QuickSort_int (array1, p, q, array2);
		QuickSort_int (array1, (q + 1), r, array2);
	}
}

int Partition_int (int array1 [ARRAY_SIZE], int p, int r, long int array2 [ARRAY_SIZE])
{
	int x, temp;
	int i, j;
	long int key;

	x = array1 [p];
	i = p - 1;
	j = r + 1;
	
	while (1) {
		do {
			j--;
		} while (array1 [j] > x);
		do {
			i++;
		} while (array1 [i] < x);
		if (i < j) {
			temp = array1 [i];
			array1 [i] = array1 [j];
			array1 [j] = temp;
			key = array2 [i];
			array2 [i] = array2 [j];
			array2 [j] = key;
		}
		else {
			break;
		}
	}

	return j;
}

/*********************************************************/

void QuickSort_double (double array1 [ARRAY_SIZE], int p, int r, KEY_TYPE array2 [ARRAY_SIZE])
{
	int q;

	if (p < r) {
		q = Partition_double (array1, p, r, array2);
		QuickSort_double (array1, p, q, array2);
		QuickSort_double (array1, (q + 1), r, array2);
	}
}

int Partition_double (double array1 [ARRAY_SIZE], int p, int r, KEY_TYPE array2 [ARRAY_SIZE])
{
	double x, temp;
	int i, j;
	long int key;

	x = array1 [p];
	i = p - 1;
	j = r + 1;
	
	while (1) {
		do {
			j--;
		} while (array1 [j] < x);
		do {
			i++;
		} while (array1 [i] > x);
		if (i < j) {
			temp = array1 [i];
			array1 [i] = array1 [j];
			array1 [j] = temp;
			key = array2 [i];
			array2 [i] = array2 [j];
			array2 [j] = key;
		}
		else {
			break;
		}
	}

	return j;
}